#
# Copyright (c) 2014, Arista Networks, Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
#   Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
#   Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
#
#   Neither the name of Arista Networks nor the names of its
#   contributors may be used to endorse or promote products derived from
#   this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL ARISTA NETWORKS
# BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
# BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
# WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
# OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
# IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
import sys
import os
import unittest

sys.path.append(os.path.join(os.path.dirname(__file__), '../lib'))

from testlib import get_fixture, function
from testlib import EapiConfigUnitTest

import pyeapi.api.acl

class TestApiAclFunctions(unittest.TestCase):

    def test_mask_to_prefixlen(self):
        result = pyeapi.api.acl.mask_to_prefixlen('255.255.255.0')
        self.assertEqual(result, 24)

    def test_prefixlen_to_mask(self):
        result = pyeapi.api.acl.prefixlen_to_mask(24)
        self.assertEqual(result, '255.255.255.0')


class TestApiAcls(EapiConfigUnitTest):

    def __init__(self, *args, **kwargs):
        super(TestApiAcls, self).__init__(*args, **kwargs)
        self.instance = pyeapi.api.acl.Acls(None)
        self.config = open(get_fixture('running_config.text')).read()

    def test_instance(self):
        result = pyeapi.api.acl.instance(None)
        self.assertIsInstance(result, pyeapi.api.acl.Acls)

    def test_getall(self):
        result = self.instance.getall()
        self.assertIsInstance(result, dict)
        self.assertIn('exttest', result['extended'])
        self.assertIn('test', result['standard'])

    def test_get_not_configured(self):
        self.assertIsNone(self.instance.get('unconfigured'))

    def test_get(self):
        result = self.instance.get('test')
        keys = ['name', 'type', 'entries']
        self.assertEqual(sorted(keys), sorted(result.keys()))

    def test_get_instance(self):
        result = self.instance.get_instance('test')
        self.assertIsInstance(result, pyeapi.api.acl.StandardAcls)
        self.instance._instances['test'] = result
        result = self.instance.get_instance('exttest')
        self.assertIsInstance(result, pyeapi.api.acl.ExtendedAcls)
        result = self.instance.get_instance('unconfigured')
        self.assertIsInstance(result, dict)
        self.assertIsNone(result['unconfigured'])
        result = self.instance.get_instance('test')
        self.assertIsInstance(result, pyeapi.api.acl.StandardAcls)
        self.assertEqual(len(self.instance._instances), 2)

    def test_create_instance_standard(self):
        result = self.instance.create_instance('test', 'standard')
        self.assertIsInstance(result, pyeapi.api.acl.StandardAcls)
        self.assertEqual(len(self.instance._instances), 1)

    def test_create_instance_extended(self):
        result = self.instance.create_instance('exttest', 'extended')
        self.assertIsInstance(result, pyeapi.api.acl.ExtendedAcls)
        self.assertEqual(len(self.instance._instances), 1)

    def test_create_standard(self):
        cmds = 'ip access-list standard test'
        func = function('create', 'test')
        self.eapi_positive_config_test(func, cmds)

    def test_create_extended(self):
        cmds = 'ip access-list exttest'
        func = function('create', 'exttest', 'extended')
        self.eapi_positive_config_test(func, cmds)

    def test_create_unknown_type_creates_standard(self):
        cmds = 'ip access-list standard test'
        func = function('create', 'test', 'unknown')
        self.eapi_positive_config_test(func, cmds)

    def test_proxy_method_success(self):
        result = self.instance.remove_entry('test', '10')
        self.assertTrue(result)

    def test_proxy_method_raises_attribute_error(self):
        with self.assertRaises(AttributeError):
            self.instance.nonmethod('test', '10')


class TestApiStandardAcls(EapiConfigUnitTest):

    def __init__(self, *args, **kwargs):
        super(TestApiStandardAcls, self).__init__(*args, **kwargs)
        self.instance = pyeapi.api.acl.StandardAcls(None)
        self.config = open(get_fixture('running_config.text')).read()

    def test_get(self):
        result = self.instance.get('test')
        keys = ['name', 'type', 'entries']
        self.assertEqual(sorted(keys), sorted(result.keys()))
        self.assertEqual(result['type'], 'standard')

    def test_get_not_configured(self):
        self.assertIsNone(self.instance.get('unconfigured'))

    def test_acl_functions(self):
        for name in ['create', 'delete', 'default']:
            if name == 'create':
                cmds = 'ip access-list standard test'
            elif name == 'delete':
                cmds = 'no ip access-list standard test'
            elif name == 'default':
                cmds = 'default ip access-list standard test'
            func = function(name, 'test')
            self.eapi_positive_config_test(func, cmds)

    def test_update_entry(self):
        cmds = ['ip access-list standard test', 'no 10',
                '10 permit 0.0.0.0/32 log', 'exit']
        func = function('update_entry', 'test', '10', 'permit', '0.0.0.0',
                        '32', True)
        self.eapi_positive_config_test(func, cmds)

    def test_update_entry_no_log(self):
        cmds = ['ip access-list standard test', 'no 10',
                '10 permit 0.0.0.0/32', 'exit']
        func = function('update_entry', 'test', '10', 'permit', '0.0.0.0',
                        '32')
        self.eapi_positive_config_test(func, cmds)

    def test_remove_entry(self):
        cmds = ['ip access-list standard test', 'no 10', 'exit']
        func = function('remove_entry', 'test', '10')
        self.eapi_positive_config_test(func, cmds)

    def test_add_entry(self):
        cmds = ['ip access-list standard test', 'permit 0.0.0.0/32 log',
                'exit']
        func = function('add_entry', 'test', 'permit', '0.0.0.0',
                        '32', True)
        self.eapi_positive_config_test(func, cmds)

    def test_add_entry_no_log(self):
        cmds = ['ip access-list standard test', 'permit 0.0.0.0/32',
                'exit']
        func = function('add_entry', 'test', 'permit', '0.0.0.0',
                        '32')
        self.eapi_positive_config_test(func, cmds)

    def test_add_entry_with_seqno(self):
        cmds = ['ip access-list standard test', '30 permit 0.0.0.0/32 log',
                'exit']
        func = function('add_entry', 'test', 'permit', '0.0.0.0',
                        '32', True, 30)
        self.eapi_positive_config_test(func, cmds)


class TestApiExtendedAcls(EapiConfigUnitTest):

    def __init__(self, *args, **kwargs):
        super(TestApiExtendedAcls, self).__init__(*args, **kwargs)
        self.instance = pyeapi.api.acl.ExtendedAcls(None)
        self.config = open(get_fixture('running_config.text')).read()

    def test_get(self):
        result = self.instance.get('exttest')
        keys = ['name', 'type', 'entries']
        self.assertEqual(sorted(keys), sorted(result.keys()))
        self.assertEqual(result['type'], 'extended')
        self.assertIn('entries', result)
        self.assertIn('50', result['entries'])
        entry = dict(action='permit', dstaddr='1.1.1.2', dstlen=None,
                     dstport=None, other=None, protocol='ip', srcaddr='any',
                     srclen=None, srcport=None)
        self.assertEqual(entry, result['entries']['50'])
        self.assertIn('70', result['entries'])
        entry = dict(action='permit', dstaddr='3.3.3.3', dstlen=None,
                     dstport='lt ipp', protocol='tcp', srcaddr='8.8.8.0',
                     other='urg ttl eq 24 fragments tracked log',
                     srclen='24', srcport='neq irc')
        self.assertEqual(entry, result['entries']['70'])

    def test_get_not_configured(self):
        self.assertIsNone(self.instance.get('unconfigured'))

    def test_acl_functions(self):
        for name in ['create', 'delete', 'default']:
            if name == 'create':
                cmds = 'ip access-list exttest'
            elif name == 'delete':
                cmds = 'no ip access-list exttest'
            elif name == 'default':
                cmds = 'default ip access-list exttest'
            func = function(name, 'exttest')
            self.eapi_positive_config_test(func, cmds)

    def test_update_entry(self):
        cmds = ['ip access-list exttest', 'no 10',
                '10 permit ip 0.0.0.0/32 1.1.1.1/32 log', 'exit']
        func = function('update_entry', 'exttest', '10', 'permit', 'ip',
                        '0.0.0.0', '32', '1.1.1.1', '32', True)
        self.eapi_positive_config_test(func, cmds)

    def test_update_entry_no_log(self):
        cmds = ['ip access-list exttest', 'no 10',
                '10 permit ip 0.0.0.0/32 1.1.1.1/32', 'exit']
        func = function('update_entry', 'exttest', '10', 'permit', 'ip',
                        '0.0.0.0', '32', '1.1.1.1', '32')
        self.eapi_positive_config_test(func, cmds)

    def test_remove_entry(self):
        cmds = ['ip access-list exttest', 'no 10', 'exit']
        func = function('remove_entry', 'exttest', '10')
        self.eapi_positive_config_test(func, cmds)

    def test_add_entry(self):
        cmds = ['ip access-list exttest',
                'permit ip 0.0.0.0/32 1.1.1.1/32 log', 'exit']
        func = function('add_entry', 'exttest', 'permit', 'ip', '0.0.0.0',
                        '32', '1.1.1.1', '32', True)
        self.eapi_positive_config_test(func, cmds)

    def test_add_entry_no_log(self):
        cmds = ['ip access-list exttest', 'permit ip 0.0.0.0/32 1.1.1.1/32',
                'exit']
        func = function('add_entry', 'exttest', 'permit', 'ip', '0.0.0.0',
                        '32', '1.1.1.1', '32')
        self.eapi_positive_config_test(func, cmds)

    def test_add_entry_with_seqno(self):
        cmds = ['ip access-list exttest',
                '30 permit ip 0.0.0.0/32 1.1.1.1/32 log', 'exit']
        func = function('add_entry', 'exttest', 'permit', 'ip', '0.0.0.0',
                        '32', '1.1.1.1', '32', True, 30)
        self.eapi_positive_config_test(func, cmds)


if __name__ == '__main__':
    unittest.main()



#
# right (c) 2015, Arista Networks, Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
#   Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
#   Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
#
#   Neither the name of Arista Networks nor the names of its
#   contributors may be used to endorse or promote products derived from
#   this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL ARISTA NETWORKS
# BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
# BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
# WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
# OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
# IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
import sys
import os
import unittest

sys.path.append(os.path.join(os.path.dirname(__file__), '../lib'))

from testlib import get_fixture, function
from testlib import EapiConfigUnitTest

import pyeapi.api.vrrp

upd_intf = 'Vlan50'
upd_vrid = 10
upd_cmd = 'interface %s' % upd_intf
known_vrrps = {
    'Ethernet1': {
        10: {'priority': 175,
             'timers_advertise': 1,
             'mac_addr_adv_interval': 30,
             'preempt': True,
             'preempt_delay_min': 0,
             'preempt_delay_reload': 0,
             'delay_reload': 0,
             'primary_ip': '10.10.6.10',
             'secondary_ip': [],
             'description': 'vrrp 10 on Ethernet1',
             'enable': True,
             'track': [],
             'bfd_ip': '',
             'ip_version': 2}
    },
    'Port-Channel10': {
        10: {'priority': 150,
             'timers_advertise': 1,
             'mac_addr_adv_interval': 30,
             'preempt': True,
             'preempt_delay_min': 0,
             'preempt_delay_reload': 0,
             'delay_reload': 0,
             'primary_ip': '10.10.5.10',
             'secondary_ip': ['10.10.5.20'],
             'description': 'vrrp 10 on Port-Channel10',
             'enable': True,
             'track': [],
             'bfd_ip': '',
             'ip_version': 2}
    },
    'Vlan50': {
        10: {'priority': 200,
             'timers_advertise': 3,
             'mac_addr_adv_interval': 30,
             'preempt': True,
             'preempt_delay_min': 0,
             'preempt_delay_reload': 0,
             'delay_reload': 0,
             'primary_ip': '10.10.4.10',
             'secondary_ip': ['10.10.4.21', '10.10.4.22',
                              '10.10.4.23', '10.10.4.24'],
             'description': '',
             'enable': True,
             'track': [
                 {'name': 'Ethernet1', 'action': 'decrement', 'amount': 10},
                 {'name': 'Ethernet1', 'action': 'shutdown'},
                 {'name': 'Ethernet2', 'action': 'decrement', 'amount': 50},
                 {'name': 'Ethernet2', 'action': 'shutdown'},
                 {'name': 'Ethernet11', 'action': 'decrement', 'amount': 75},
                 {'name': 'Ethernet11', 'action': 'shutdown'},
             ],
             'bfd_ip': '',
             'ip_version': 2},
        20: {'priority': 100,
             'timers_advertise': 5,
             'mac_addr_adv_interval': 30,
             'preempt': False,
             'preempt_delay_min': 0,
             'preempt_delay_reload': 0,
             'delay_reload': 0,
             'primary_ip': '10.10.4.20',
             'secondary_ip': [],
             'description': '',
             'enable': False,
             'track': [
                 {'name': 'Ethernet1', 'action': 'shutdown'},
                 {'name': 'Ethernet2', 'action': 'decrement', 'amount': 1},
                 {'name': 'Ethernet2', 'action': 'shutdown'},
             ],
             'bfd_ip': '',
             'ip_version': 2},
        30: {'priority': 50,
             'timers_advertise': 1,
             'mac_addr_adv_interval': 30,
             'preempt': True,
             'preempt_delay_min': 0,
             'preempt_delay_reload': 0,
             'delay_reload': 0,
             'primary_ip': '10.10.4.30',
             'secondary_ip': [],
             'description': '',
             'enable': True,
             'track': [],
             'bfd_ip': '10.10.4.33',
             'ip_version': 2}
    }
}


class TestApiVrrp(EapiConfigUnitTest):

    maxDiff = None

    def __init__(self, *args, **kwargs):
        super(TestApiVrrp, self).__init__(*args, **kwargs)
        self.instance = pyeapi.api.vrrp.Vrrp(None)
        self.config = open(get_fixture('running_config.vrrp')).read()

    def test_instance(self):
        result = pyeapi.api.vrrp.instance(None)
        self.assertIsInstance(result, pyeapi.api.vrrp.Vrrp)

    def test_get(self):
        # Request various sets of vrrp configurations
        for interface in known_vrrps:
            known = known_vrrps.get(interface)
            for vrid in known:
                known[vrid] = self.instance.vrconf_format(known[vrid])
            result = self.instance.get(interface)
            self.assertEqual(result, known)

    def test_get_non_existent_interface(self):
        # Request vrrp configuration for an interface that
        # is not defined
        result = self.instance.get('Vlan2000')
        self.assertIsNone(result)

    def test_get_invalid_parameters(self):
        # Pass empty, None, or other invalid parameters to get()
        with self.assertRaises(ValueError):
            self.instance.get('')
        with self.assertRaises(ValueError):
            self.instance.get(None)

    def test_getall(self):
        # Get all the vrrp configurations from the config
        result = self.instance.getall()
        self.assertEqual(result, known_vrrps)

    def test_create(self):
        interface = 'Ethernet1'
        vrid = 10

        # Test create with a normal configuration
        configuration = {
            'primary_ip': '10.10.60.10',
            'priority': 200,
            'description': 'modified vrrp 10 on Ethernet1',
            'secondary_ip': ['10.10.60.20', '10.10.60.30'],
            'ip_version': 3,
            'timers_advertise': 2,
            'mac_addr_adv_interval': 3,
            'preempt': True,
            'preempt_delay_min': 1,
            'preempt_delay_reload': 1,
            'delay_reload': 1,
            'track': [
                {'name': 'Ethernet1', 'action': 'decrement', 'amount': 1},
                {'name': 'Ethernet1', 'action': 'shutdown'},
                {'name': 'Ethernet2', 'action': 'decrement', 'amount': 1},
                {'name': 'Ethernet2', 'action': 'shutdown'},
            ],
            'bfd_ip': '10.10.60.30',
        }

        cmds = [
            'interface Ethernet1',
            'vrrp 10 shutdown',
            'vrrp 10 ip 10.10.60.10',
            'vrrp 10 priority 200',
            'vrrp 10 description modified vrrp 10 on Ethernet1',
            'vrrp 10 ip version 3',
            'vrrp 10 ip 10.10.60.20 secondary',
            'vrrp 10 ip 10.10.60.30 secondary',
            'vrrp 10 timers advertise 2',
            'vrrp 10 mac-address advertisement-interval 3',
            'vrrp 10 preempt',
            'vrrp 10 preempt delay minimum 1',
            'vrrp 10 preempt delay reload 1',
            'vrrp 10 delay reload 1',
            'vrrp 10 track Ethernet1 decrement 1',
            'vrrp 10 track Ethernet1 shutdown',
            'vrrp 10 track Ethernet2 decrement 1',
            'vrrp 10 track Ethernet2 shutdown',
            'vrrp 10 bfd ip 10.10.60.30',
        ]
        func = function('create', interface, vrid, **configuration)

        self.eapi_positive_config_test(func, cmds)

        # Test create setting possible parameters to 'no'
        configuration = {
            'primary_ip': 'no',
            'priority': 'no',
            'description': 'no',
            'secondary_ip': [],
            'ip_version': 'no',
            'timers_advertise': 'no',
            'mac_addr_adv_interval': 'no',
            'preempt': 'no',
            'preempt_delay_min': 'no',
            'preempt_delay_reload': 'no',
            'delay_reload': 'no',
            'track': [],
            'bfd_ip': 'no',
        }

        cmds = [
            'interface Ethernet1',
            'vrrp 10 shutdown',
            'no vrrp 10 ip 10.10.6.10',
            'no vrrp 10 priority',
            'no vrrp 10 description',
            'no vrrp 10 ip version',
            'no vrrp 10 timers advertise',
            'no vrrp 10 mac-address advertisement-interval',
            'no vrrp 10 preempt',
            'no vrrp 10 preempt delay minimum',
            'no vrrp 10 preempt delay reload',
            'no vrrp 10 delay reload',
            'no vrrp 10 bfd ip',
        ]
        func = function('create', interface, vrid, **configuration)

        self.eapi_positive_config_test(func, cmds)

        # Test create setting possible parameters to 'default'
        configuration = {
            'primary_ip': 'default',
            'priority': 'default',
            'description': 'default',
            'secondary_ip': [],
            'ip_version': 'default',
            'timers_advertise': 'default',
            'mac_addr_adv_interval': 'default',
            'preempt': 'default',
            'preempt_delay_min': 'default',
            'preempt_delay_reload': 'default',
            'delay_reload': 'default',
            'track': [],
            'bfd_ip': 'default',
        }

        cmds = [
            'interface Ethernet1',
            'vrrp 10 shutdown',
            'default vrrp 10 ip 10.10.6.10',
            'default vrrp 10 priority',
            'default vrrp 10 description',
            'default vrrp 10 ip version',
            'default vrrp 10 timers advertise',
            'default vrrp 10 mac-address advertisement-interval',
            'default vrrp 10 preempt',
            'default vrrp 10 preempt delay minimum',
            'default vrrp 10 preempt delay reload',
            'default vrrp 10 delay reload',
            'default vrrp 10 bfd ip',
        ]
        func = function('create', interface, vrid, **configuration)

        self.eapi_positive_config_test(func, cmds)

    def test_delete(self):
        interface = 'Ethernet1'
        vrid = 10

        cmds = [
            'interface Ethernet1',
            'no vrrp 10',
        ]

        func = function('delete', interface, vrid)

        self.eapi_positive_config_test(func, cmds)

    def test_default(self):
        interface = 'Ethernet1'
        vrid = 10

        cmds = [
            'interface Ethernet1',
            'default vrrp 10',
        ]

        func = function('default', interface, vrid)

        self.eapi_positive_config_test(func, cmds)

    def test_set_enable(self):
        # no vrrp 10 shutdown

        # Test set_enable gives properly formatted commands
        cases = [
            (False, 'vrrp %d shutdown' % upd_vrid),
            (True, 'no vrrp %d shutdown' % upd_vrid),
        ]

        for (enable, cmd) in cases:
            func = function('set_enable', upd_intf, upd_vrid, value=enable)
            exp_cmds = [upd_cmd] + [cmd]
            self.eapi_positive_config_test(func, exp_cmds)

        # Test raising ValueError from invalid parameters
        cases = ['a', 200]

        for enable in cases:
            func = function('set_enable', upd_intf, upd_vrid, value=enable)
            self.eapi_exception_config_test(func, ValueError)

    def test_set_primary_ip(self):
        # vrrp 10 ip 10.10.4.10

        # Test set_primary_ip gives properly formatted commands
        ip1 = '10.10.4.110'
        ipcurr = '10.10.4.10'
        cases = [
            (ip1, None, None, 'vrrp %d ip %s' % (upd_vrid, ip1)),
            (ip1, True, None, 'no vrrp %d ip %s' % (upd_vrid, ipcurr)),
            (ip1, None, True, 'default vrrp %d ip %s' % (upd_vrid, ipcurr)),
            (ip1, True, True, 'default vrrp %d ip %s' % (upd_vrid, ipcurr)),
        ]

        for (primary_ip, disable, default, cmd) in cases:
            func = function('set_primary_ip', upd_intf, upd_vrid,
                            value=primary_ip, disable=disable, default=default)
            exp_cmds = [upd_cmd] + [cmd]
            self.eapi_positive_config_test(func, exp_cmds)

        # Test raising ValueError from invalid parameters
        cases = ['abc', 500, '101.101']

        for primary_ip in cases:
            func = function('set_primary_ip', upd_intf, upd_vrid,
                            value=primary_ip)
            self.eapi_exception_config_test(func, ValueError)

    def test_set_priority(self):
        # vrrp 10 priority 200

        # Test set_primary_ip gives properly formatted commands
        cases = [
            (150, None, None, 'vrrp %d priority 150' % upd_vrid),
            (None, None, True, 'default vrrp %d priority' % upd_vrid),
            (None, True, True, 'default vrrp %d priority' % upd_vrid),
            (None, True, None, 'no vrrp %d priority' % upd_vrid),
        ]

        for (priority, disable, default, cmd) in cases:
            func = function('set_priority', upd_intf, upd_vrid,
                            value=priority, disable=disable, default=default)
            exp_cmds = [upd_cmd] + [cmd]
            self.eapi_positive_config_test(func, exp_cmds)

        # Test raising ValueError from invalid parameters
        cases = ['abc', 500, False]

        for priority in cases:
            func = function('set_priority', upd_intf, upd_vrid,
                            value=priority)
            self.eapi_exception_config_test(func, ValueError)

    def test_set_description(self):
        # no vrrp 10 description

        desc = 'test description'

        # Test set_description gives properly formatted commands
        cases = [
            (desc, None, None, 'vrrp %d description %s' % (upd_vrid, desc)),
            (None, None, True, 'default vrrp %d description' % upd_vrid),
            (None, True, True, 'default vrrp %d description' % upd_vrid),
            (None, True, None, 'no vrrp %d description' % upd_vrid),
        ]

        for (description, disable, default, cmd) in cases:
            func = function('set_description', upd_intf, upd_vrid,
                            value=description, disable=disable,
                            default=default)
            exp_cmds = [upd_cmd] + [cmd]
            self.eapi_positive_config_test(func, exp_cmds)

    def test_set_ip_version(self):
        # vrrp 10 ip version 2
        # Test set_description gives properly formatted commands
        cases = [
            (2, None, None, 'vrrp %d ip version 2' % upd_vrid),
            (None, None, True, 'default vrrp %d ip version' % upd_vrid),
            (None, True, True, 'default vrrp %d ip version' % upd_vrid),
            (None, True, None, 'no vrrp %d ip version' % upd_vrid),
        ]

        for (ip_version, disable, default, cmd) in cases:
            func = function('set_ip_version', upd_intf, upd_vrid,
                            value=ip_version, disable=disable, default=default)
            exp_cmds = [upd_cmd] + [cmd]
            self.eapi_positive_config_test(func, exp_cmds)

        # Test raising ValueError by entering invalid parameters
        cases = ['a', 5]

        for ip_version in cases:
            func = function('set_ip_version', upd_intf, upd_vrid,
                            value=ip_version)
            self.eapi_exception_config_test(func, ValueError)

    def test_set_secondary_ips(self):
        # vrrp 10 ip 10.10.4.21 secondary
        # vrrp 10 ip 10.10.4.22 secondary
        # vrrp 10 ip 10.10.4.23 secondary

        curr1 = '10.10.4.21'
        curr2 = '10.10.4.22'
        curr3 = '10.10.4.23'
        curr4 = '10.10.4.24'

        new1 = '10.10.4.31'
        new2 = '10.10.4.32'
        new3 = '10.10.4.33'
        new4 = curr4

        # Test set_secondary_ips gives properly formatted commands
        cases = [
            ([new1, new2, new3],
             {'add': [new1, new2, new3],
              'remove': [curr1, curr2, curr3, curr4]}),
            ([new1, new2, new4],
             {'add': [new1, new2],
              'remove': [curr1, curr2, curr3]}),
            ([],
             {'add': [],
              'remove': [curr1, curr2, curr3, curr4]}),
        ]

        for (secondary_ips, cmd_dict) in cases:
            cmds = []
            for sec_ip in cmd_dict['add']:
                cmds.append("vrrp %d ip %s secondary" % (upd_vrid, sec_ip))

            for sec_ip in cmd_dict['remove']:
                cmds.append("no vrrp %d ip %s secondary" % (upd_vrid, sec_ip))

            func = function('set_secondary_ips', upd_intf, upd_vrid,
                            secondary_ips)
            exp_cmds = [upd_cmd] + sorted(cmds)
            self.eapi_positive_config_test(func, exp_cmds)

        # Test raising ValueError by entering invalid parameters
        cases = [
            [new1, new2, 'abc'],
            [new1, new2, '10.10.10'],
            [new1, new2, True],
        ]

        for secondary_ips in cases:
            func = function('set_secondary_ips', upd_intf, upd_vrid,
                            secondary_ips)
            self.eapi_exception_config_test(func, ValueError)

    def test_set_timers_advertise(self):
        # vrrp 10 timers advertise 3

        # Test set_timers_advertise gives properly formatted commands
        cases = [
            (50, None, None, 'vrrp %d timers advertise 50' % upd_vrid),
            (None, None, True, 'default vrrp %d timers advertise' % upd_vrid),
            (None, True, True, 'default vrrp %d timers advertise' % upd_vrid),
            (None, True, None, 'no vrrp %d timers advertise' % upd_vrid),
        ]

        for (timers_advertise, disable, default, cmd) in cases:
            func = function('set_timers_advertise', upd_intf, upd_vrid,
                            value=timers_advertise, disable=disable,
                            default=default)
            exp_cmds = [upd_cmd] + [cmd]
            self.eapi_positive_config_test(func, exp_cmds)

        # Test raising ValueError by entering invalid parameters
        cases = [256, 0, 'a']

        for timers_advertise in cases:
            func = function('set_timers_advertise', upd_intf, upd_vrid,
                            value=timers_advertise)
            self.eapi_exception_config_test(func, ValueError)

    def test_set_mac_addr_adv_interval(self):
        # vrrp 10 mac-address advertisement-interval 30

        # Test set_timers_advertise gives properly formatted commands
        maadvint = 'mac-address advertisement-interval'
        cases = [
            (50, None, None, 'vrrp %d %s 50' % (upd_vrid, maadvint)),
            (None, None, True, 'default vrrp %d %s' % (upd_vrid, maadvint)),
            (None, True, True, 'default vrrp %d %s' % (upd_vrid, maadvint)),
            (None, True, None, 'no vrrp %d %s' % (upd_vrid, maadvint)),
        ]

        for (mac_addr_adv_interval, disable, default, cmd) in cases:
            func = function('set_mac_addr_adv_interval', upd_intf, upd_vrid,
                            value=mac_addr_adv_interval, disable=disable,
                            default=default)
            exp_cmds = [upd_cmd] + [cmd]
            self.eapi_positive_config_test(func, exp_cmds)

        # Test raising ValueError by entering invalid parameters
        cases = ['a', 10000]

        for mac_addr_adv_interval in cases:
            func = function('set_mac_addr_adv_interval', upd_intf, upd_vrid,
                            value=mac_addr_adv_interval)
            self.eapi_exception_config_test(func, ValueError)

    def test_set_preempt(self):
        # vrrp 10 preempt

        # Test set_description gives properly formatted commands
        cases = [
            (False, None, None, 'no vrrp %d preempt' % upd_vrid),
            (True, None, None, 'vrrp %d preempt' % upd_vrid),
            (None, None, True, 'default vrrp %d preempt' % upd_vrid),
            (None, True, True, 'default vrrp %d preempt' % upd_vrid),
            (None, True, None, 'no vrrp %d preempt' % upd_vrid),
        ]

        for (preempt, disable, default, cmd) in cases:
            func = function('set_preempt', upd_intf, upd_vrid,
                            value=preempt, disable=disable, default=default)
            exp_cmds = [upd_cmd] + [cmd]
            self.eapi_positive_config_test(func, exp_cmds)

        # Test raising ValueError by entering invalid parameters
        cases = ['a', 5]

        for preempt in cases:
            func = function('set_preempt', upd_intf, upd_vrid,
                            value=preempt)
            self.eapi_exception_config_test(func, ValueError)

    def test_set_preempt_delay_min(self):
        # vrrp 10 preempt delay minimum 0

        # Test set_preempt_delay_min gives properly formatted commands
        cases = [
            (2500, None, None,
             'vrrp %d preempt delay minimum 2500' % upd_vrid),
            (None, None, True,
             'default vrrp %d preempt delay minimum' % upd_vrid),
            (None, True, True,
             'default vrrp %d preempt delay minimum' % upd_vrid),
            (None, True, None, 'no vrrp %d preempt delay minimum' % upd_vrid),
        ]

        for (preempt_delay_min, disable, default, cmd) in cases:
            func = function('set_preempt_delay_min', upd_intf, upd_vrid,
                            value=preempt_delay_min, disable=disable,
                            default=default)
            exp_cmds = [upd_cmd] + [cmd]
            self.eapi_positive_config_test(func, exp_cmds)

        # Test raising ValueError by entering invalid parameters
        cases = ['a', 3601]

        for preempt_delay_min in cases:
            func = function('set_preempt_delay_min', upd_intf, upd_vrid,
                            value=preempt_delay_min)
            self.eapi_exception_config_test(func, ValueError)

    def test_set_preempt_delay_reload(self):
        # vrrp 10 preempt delay reload 0

        # Test set_preempt_delay_min gives properly formatted commands
        cases = [
            (1500, None, None,
             'vrrp %d preempt delay reload 1500' % upd_vrid),
            (None, None, True,
             'default vrrp %d preempt delay reload' % upd_vrid),
            (None, True, True,
             'default vrrp %d preempt delay reload' % upd_vrid),
            (None, True, None, 'no vrrp %d preempt delay reload' % upd_vrid),
        ]

        for (preempt_delay_reload, disable, default, cmd) in cases:
            func = function('set_preempt_delay_reload', upd_intf, upd_vrid,
                            value=preempt_delay_reload, disable=disable,
                            default=default)
            exp_cmds = [upd_cmd] + [cmd]
            self.eapi_positive_config_test(func, exp_cmds)

        # Test raising ValueError by entering invalid parameters
        cases = ['a', 3601]

        for preempt_delay_reload in cases:
            func = function('set_preempt_delay_reload', upd_intf, upd_vrid,
                            value=preempt_delay_reload)
            self.eapi_exception_config_test(func, ValueError)

    def test_set_delay_reload(self):
        # vrrp 10 delay reload 0

        # Test set_delay_min gives properly formatted commands
        cases = [
            (1750, None, None, 'vrrp %d delay reload 1750' % upd_vrid),
            (None, None, True, 'default vrrp %d delay reload' % upd_vrid),
            (None, True, True, 'default vrrp %d delay reload' % upd_vrid),
            (None, True, None, 'no vrrp %d delay reload' % upd_vrid),
        ]

        for (delay_reload, disable, default, cmd) in cases:
            func = function('set_delay_reload', upd_intf, upd_vrid,
                            value=delay_reload, disable=disable,
                            default=default)
            exp_cmds = [upd_cmd] + [cmd]
            self.eapi_positive_config_test(func, exp_cmds)

        # Test raising ValueError by entering invalid parameters
        cases = ['a', 3601]

        for delay_reload in cases:
            func = function('set_delay_reload', upd_intf, upd_vrid,
                            value=delay_reload)
            self.eapi_exception_config_test(func, ValueError)

    def test_set_tracks(self):
        # vrrp 10 track Ethernet1 decrement 10
        # vrrp 10 track Ethernet1 shutdown
        # vrrp 10 track Ethernet2 decrement 50
        # vrrp 10 track Ethernet2 shutdown
        # vrrp 10 track Ethernet11 decrement 75
        # vrrp 10 track Ethernet11 shutdown

        curr1 = {'name': 'Ethernet1', 'action': 'decrement', 'amount': 10}
        curr2 = {'name': 'Ethernet1', 'action': 'shutdown'}
        curr3 = {'name': 'Ethernet2', 'action': 'decrement', 'amount': 50}
        curr4 = {'name': 'Ethernet2', 'action': 'shutdown'}
        curr5 = {'name': 'Ethernet11', 'action': 'decrement', 'amount': 75}
        curr6 = {'name': 'Ethernet11', 'action': 'shutdown'}

        new1 = curr1
        new2 = {'name': 'Ethernet2', 'action': 'decrement', 'amount': 49}
        new3 = {'name': 'Ethernet3', 'action': 'shutdown'}
        new4 = {'name': 'Ethernet4', 'action': 'decrement', 'amount': 50}
        new5 = {'name': 'Ethernet5', 'action': 'shutdown'}
        new6 = {'name': 'Ethernet9', 'action': 'decrement', 'amount': 75}

        # Test set_track gives properly formatted commands
        cases = [
            ([curr6, curr5, new1, new2],
             {'add': [new2],
              'remove': [curr2, curr3, curr4]}),
            ([new2, new3, new4, new5, new6],
             {'add': [new2, new3, new4, new5, new6],
              'remove': [curr1, curr2, curr3, curr4, curr5, curr6]}),
            ([],
             {'add': [],
              'remove': [curr1, curr2, curr3, curr4, curr5, curr6]}),
        ]

        for (tracks, cmd_dict) in cases:
            cmds = []
            for add in cmd_dict['add']:
                tr_obj = add['name']
                action = add['action']
                amount = add['amount'] if 'amount' in add else ''
                cmd = ("vrrp %d track %s %s %s"
                       % (upd_vrid, tr_obj, action, amount))
                cmds.append(cmd.rstrip())

            for remove in cmd_dict['remove']:
                tr_obj = remove['name']
                action = remove['action']
                amount = remove['amount'] if 'amount' in remove else ''
                cmd = ("no vrrp %d track %s %s %s"
                       % (upd_vrid, tr_obj, action, amount))
                cmds.append(cmd.rstrip())

            func = function('set_tracks', upd_intf, upd_vrid, tracks)
            exp_cmds = [upd_cmd] + sorted(cmds)
            self.eapi_positive_config_test(func, exp_cmds)

        # Test raising ValueError by entering invalid parameters
        cases = [
            [{'name': 'Ethernet1', 'action': 'disable', 'amount': 10}],
            [{'name': 'Ethernet1', 'action': 'decrement', 'amount': True}],
            [{'name': 'Ethernet1', 'action': 'shutdown', 'amount': 10}],
            [{'action': 'decrement', 'amount': 10}],
            [{'name': 'Ethernet1', 'action': 'decrement',
              'amount': 10, 'bad': 1}],
        ]

        for tracks in cases:
            func = function('set_tracks', upd_intf, upd_vrid, tracks)
            self.eapi_exception_config_test(func, ValueError)

    def test_set_bfd_ip(self):
        # no vrrp 10 bfd ip

        bfd_addr = '10.10.4.101'

        # Test bfd_ip gives properly formatted commands
        cases = [
            (bfd_addr, None, None, 'vrrp %d bfd ip %s' % (upd_vrid, bfd_addr)),
            (None, True, None, 'no vrrp %d bfd ip' % upd_vrid),
            (None, None, True, 'default vrrp %d bfd ip' % upd_vrid),
            (None, True, True, 'default vrrp %d bfd ip' % upd_vrid),
        ]

        for (bfd_ip, disable, default, cmd) in cases:
            func = function('set_bfd_ip', upd_intf, upd_vrid,
                            value=bfd_ip, disable=disable, default=default)
            exp_cmds = [upd_cmd] + [cmd]
            self.eapi_positive_config_test(func, exp_cmds)

        # Test raising ValueError from invalid parameters
        cases = ['abc', 500, '101.101']

        for bfd_ip in cases:
            func = function('set_bfd_ip', upd_intf, upd_vrid,
                            value=bfd_ip)
            self.eapi_exception_config_test(func, ValueError)

    def test_vrconf_format(self):
        # Test the function to format a vrrp configuration to
        # match the output from get/getall
        vrconf = {
            'priority': None,
            'timers_advertise': None,
            'mac_addr_adv_interval': None,
            'preempt': 'default',
            'preempt_delay_min': None,
            'preempt_delay_reload': None,
            'delay_reload': None,
            'primary_ip': None,
            'secondary_ip': ['10.10.4.22', '10.10.4.21'],
            'description': None,
            'enable': True,
            'track': [
                {'name': 'Ethernet1', 'action': 'shutdown'},
                {'name': 'Ethernet1', 'action': 'decrement', 'amount': 10},
            ],
            'bfd_ip': None,
            'ip_version': None}

        fixed = {
            'priority': 100,
            'timers_advertise': 1,
            'mac_addr_adv_interval': 30,
            'preempt': False,
            'preempt_delay_min': 0,
            'preempt_delay_reload': 0,
            'delay_reload': 0,
            'primary_ip': '0.0.0.0',
            'secondary_ip': ['10.10.4.21', '10.10.4.22'],
            'description': None,
            'enable': True,
            'track': [
                {'name': 'Ethernet1', 'action': 'decrement', 'amount': 10},
                {'name': 'Ethernet1', 'action': 'shutdown'},
            ],
            'bfd_ip': '',
            'ip_version': 2}

        # Get the vrconf_format method from the library
        func = getattr(self.instance, 'vrconf_format')
        # Call the method with the vrconf dictionary
        result = func(vrconf)
        # And verify the result is a properly formatted dictionary
        self.assertEqual(fixed, result)


if __name__ == '__main__':
    unittest.main()



#
# Copyright (c) 2015, Arista Networks, Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
#   Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
#   Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
#
#   Neither the name of Arista Networks nor the names of its
#   contributors may be used to endorse or promote products derived from
#   this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL ARISTA NETWORKS
# BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
# BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
# WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
# OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
# IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
import sys
import os
import unittest

sys.path.append(os.path.join(os.path.dirname(__file__), '../lib'))

from testlib import get_fixture, random_string, function
from testlib import EapiConfigUnitTest

import pyeapi.api.system

class TestApiSystem(EapiConfigUnitTest):

    def __init__(self, *args, **kwargs):
        super(TestApiSystem, self).__init__(*args, **kwargs)
        self.instance = pyeapi.api.system.instance(None)
        self.config = open(get_fixture('running_config.text')).read()

    def test_get(self):
        keys = ['hostname', 'iprouting', 'banner_motd', 'banner_login']
        result = self.instance.get()
        self.assertEqual(sorted(keys), sorted(list(result.keys())))
        self.assertIsNotNone(self.instance.get()['banner_motd'])
        self.assertIsNotNone(self.instance.get()['banner_login'])

    def test_set_hostname(self):
        for state in ['config', 'negate', 'default']:
            value = random_string()
            if state == 'config':
                cmds = 'hostname %s' % value
                func = function('set_hostname', value)
            elif state == 'negate':
                cmds = 'no hostname'
                func = function('set_hostname', disable=True)
            elif state == 'default':
                cmds = 'default hostname'
                func = function('set_hostname', value=value, default=True)
            self.eapi_positive_config_test(func, cmds)

    def test_set_iprouting(self):
        for state in ['config', 'negate', 'default']:
            if state == 'config':
                cmds = 'ip routing'
                func = function('set_iprouting', True)
            elif state == 'negate':
                cmds = 'no ip routing'
                func = function('set_iprouting', disable=True)
            elif state == 'default':
                cmds = 'default ip routing'
                func = function('set_iprouting', default=True)
            self.eapi_positive_config_test(func, cmds)

    def test_set_banner(self):
        banner_value = random_string() + "\n"
        func = function('set_banner', banner_type='motd',
                        value=banner_value)
        cmds = [dict(cmd='banner motd', input=banner_value)]
        self.eapi_positive_config_test(func, cmds)

    def test_set_banner_default_disable(self):
        func = function('set_banner', banner_type='motd',
                        value=None, default=True)
        cmds = 'default banner motd'
        self.eapi_positive_config_test(func, cmds)
        func = function('set_banner', banner_type='motd',
                        value=None, disable=True)
        cmds = 'no banner motd'
        self.eapi_positive_config_test(func, cmds)


if __name__ == '__main__':
    unittest.main()



#
# Copyright (c) 2014, Arista Networks, Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
#   Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
#   Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
#
#   Neither the name of Arista Networks nor the names of its
#   contributors may be used to endorse or promote products derived from
#   this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL ARISTA NETWORKS
# BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
# BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
# WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
# OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
# IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
import os
import unittest

import sys
sys.path.append(os.path.join(os.path.dirname(__file__), '../lib'))

from testlib import random_string, random_int
from systestlib import DutSystemTest, random_interface


class TestResourceInterfaces(DutSystemTest):

    def test_get(self):
        for dut in self.duts:
            intf = random_interface(dut)
            dut.config(['default interface %s' % intf,
                        'interface %s' % intf,
                        'description this is a test',
                        'flowcontrol send off',
                        'flowcontrol receive on',
                        'no sflow enable'])

            result = dut.api('interfaces').get(intf)

            self.assertIsInstance(result, dict)
            self.assertEqual(result['description'], 'this is a test')
            self.assertFalse(result['shutdown'])
            self.assertEqual(result['flowcontrol_send'], 'off')
            self.assertEqual(result['flowcontrol_receive'], 'on')
            self.assertFalse(result['sflow'])

    def test_getall(self):
        for dut in self.duts:
            rintf = random_interface(dut)
            dut.config(['default interface %s' % rintf])
            result = dut.api('interfaces').getall()
            self.assertIsInstance(result, dict)
            for intf in [rintf, 'Management1']:
                self.assertIn(intf, result)

    def test_create_and_return_true(self):
        for dut in self.duts:
            dut.config('no interface Loopback0')
            result = dut.api('interfaces').create('Loopback0')
            self.assertTrue(result)
            config = dut.run_commands('show interfaces')
            self.assertIn('Loopback0', config[0]['interfaces'])

    def test_create_ethernet_raises_not_implemented_error(self):
        for dut in self.duts:
            with self.assertRaises(NotImplementedError):
                dut.api('interfaces').create(random_interface(dut))

    def test_delete_and_return_true(self):
        for dut in self.duts:
            dut.config('interface Loopback0')
            result = dut.api('interfaces').delete('Loopback0')
            self.assertTrue(result)
            config = dut.run_commands('show interfaces')
            self.assertNotIn('Loopback0', config[0]['interfaces'])

    def test_delete_ethernet_raises_not_implemented_error(self):
        for dut in self.duts:
            with self.assertRaises(NotImplementedError):
                dut.api('interfaces').delete(random_interface(dut))

    def test_create_and_delete_ethernet_sub_interface(self):
        for dut in self.duts:
            # Default Ethernet1
            dut.api('interfaces').default('Ethernet1')
            # Create subint Ethernet1.1
            res = dut.api('interfaces').create('Ethernet1.1')
            self.assertTrue(res)
            command = 'show running-config interfaces Ethernet1.1'
            output = dut.run_commands(command, encoding='text')
            self.assertIn('Ethernet1.1', output[0]['output'])
            # Delete subint Ethernet1.1
            res = dut.api('interfaces').delete('Ethernet1.1')
            self.assertTrue(res)
            output = dut.run_commands(command, encoding='text')
            self.assertEqual(output[0]['output'], '')

    def test_ethernet_set_and_unset_encapsulation(self):
        for dut in self.duts:
            # Default Ethernet1
            dut.api('interfaces').default('Ethernet1')
            # Create subint Ethernet1.1
            res = dut.api('interfaces').create('Ethernet1.1')
            self.assertTrue(res)
            # Set encapsulation
            res = dut.api('interfaces').set_encapsulation('Ethernet1.1', 4)
            self.assertTrue(res)
            command = 'show running-config interfaces Ethernet1.1'
            output = dut.run_commands(command, encoding='text')
            encap = 'encapsulation dot1q vlan 4'
            self.assertIn(encap, output[0]['output'])
            # Remove encapsulation
            res = dut.api('interfaces').set_encapsulation('Ethernet1.1', 4,
                                                          disable=True)
            self.assertTrue(res)
            output = dut.run_commands(command, encoding='text')
            self.assertNotIn(encap, output[0]['output'])
            # Delete subint Ethernet1.1
            res = dut.api('interfaces').delete('Ethernet1.1')
            self.assertTrue(res)

    def test_set_encapsulation_non_subintf_exception(self):
        for dut in self.duts:
            with self.assertRaises(NotImplementedError):
                dut.api('interfaces').set_encapsulation(random_interface(dut),
                                                        1)

    def test_set_encapsulation_non_supported_intf_exception(self):
        for dut in self.duts:
            with self.assertRaises(NotImplementedError):
                dut.api('interfaces').set_encapsulation('Vlan1234',
                                                        1)

    def test_default(self):
        for dut in self.duts:
            intf = random_interface(dut)
            dut.config(['interface %s' % intf, 'shutdown'])
            result = dut.api('interfaces').default(intf)
            self.assertTrue(result)
            config = dut.run_commands('show interfaces %s' % intf)
            config = config[0]['interfaces'][intf]
            self.assertEqual(config['interfaceStatus'], 'connected')

    def test_set_description(self):
        for dut in self.duts:
            text = random_string()
            intf = random_interface(dut)
            result = dut.api('interfaces').set_description(intf, text)
            self.assertTrue(result)
            config = dut.run_commands('show interfaces %s' % intf)
            config = config[0]['interfaces'][intf]
            self.assertEqual(config['description'], text)

    def test_set_description_negate(self):
        for dut in self.duts:
            text = random_string()
            intf = random_interface(dut)
            dut.config(['interface %s' % intf, 'description %s' % text])
            result = dut.api('interfaces').set_description(intf, disable=True)
            self.assertTrue(result)
            config = dut.run_commands('show interfaces %s' % intf)
            config = config[0]['interfaces'][intf]
            self.assertEqual(config['description'], '')

    def test_set_description_default(self):
        for dut in self.duts:
            text = random_string()
            intf = random_interface(dut)
            dut.config(['interface %s' % intf, 'description %s' % text])
            result = dut.api('interfaces').set_description(intf, default=True)
            self.assertTrue(result)
            config = dut.run_commands('show interfaces %s' % intf)
            config = config[0]['interfaces'][intf]
            self.assertEqual(config['description'], '')

    def test_set_sflow_enable(self):
        for dut in self.duts:
            intf = random_interface(dut)
            dut.config(['interface %s' % intf, 'no sflow enable'])
            result = dut.api('interfaces').set_sflow(intf, True)
            self.assertTrue(result)
            config = dut.run_commands('show running-config interfaces %s' %
                                      intf, 'text')
            self.assertNotIn('no sflow enable', config[0]['output'])

    def test_set_sflow_disable(self):
        for dut in self.duts:
            intf = random_interface(dut)
            dut.config('default interface %s' % intf)
            result = dut.api('interfaces').set_sflow(intf, disable=True)
            self.assertTrue(result)
            config = dut.run_commands('show running-config interfaces %s' %
                                      intf, 'text')
            self.assertIn('no sflow enable', config[0]['output'])

    def test_set_sflow_default(self):
        for dut in self.duts:
            intf = random_interface(dut)
            dut.config('default interface %s' % intf)
            result = dut.api('interfaces').set_sflow(intf, default=True)
            self.assertTrue(result)
            config = dut.run_commands('show running-config interfaces %s' %
                                      intf, 'text')
            self.assertNotIn('no sflow enable', config[0]['output'])

    def test_set_vrf(self):
        for dut in self.duts:
            intf = random_interface(dut)
            dut.config('default interface %s' % intf)
            # Verify set_vrf returns False if no vrf by name is configured
            result = dut.api('interfaces').set_vrf(intf, 'test')
            self.assertFalse(result)
            if dut.version_number >= '4.23':
                dut.config('vrf instance test')
            else:
                dut.config('vrf definition test')
            # Verify interface has vrf applied
            result = dut.api('interfaces').set_vrf(intf, 'test')
            self.assertTrue(result)
            config = dut.run_commands('show running-config interfaces %s' %
                                      intf, 'text')
            if dut.version_number >= '4.23':
                self.assertIn('vrf test', config[0]['output'])
            else:
                self.assertIn('vrf forwarding test', config[0]['output'])
            # Verify interface has vrf removed
            result = dut.api('interfaces').set_vrf(intf, 'test', disable=True)
            self.assertTrue(result)
            config = dut.run_commands('show running-config interfaces %s' %
                                      intf, 'text')
            if dut.version_number >= '4.23':
                self.assertIn('vrf test', config[0]['output'])
                # Remove test vrf
                dut.config('no vrf instance test')
            else:
                self.assertIn('vrf forwarding test', config[0]['output'])
                # Remove test vrf
                dut.config('no vrf definition test')


class TestPortchannelInterface(DutSystemTest):

    def test_get(self):
        for dut in self.duts:
            dut.config(['no interface Port-Channel1',
                        'interface Port-Channel1'])
            result = dut.api('interfaces').get('Port-Channel1')
            self.assertIsInstance(result, dict)
            self.assertEqual(result['type'], 'portchannel')
            self.assertEqual(result['name'], 'Port-Channel1')

    def test_set_members(self):
        for dut in self.duts:
            et1 = random_interface(dut)
            et2 = random_interface(dut, exclude=[et1])
            et3 = random_interface(dut, exclude=[et1, et2])

            dut.config(['no interface Port-Channel1',
                        'default interface %s' % et1,
                        'interface %s' % et1,
                        'channel-group 1 mode on',
                        'default interface %s' % et2,
                        'interface %s' % et2,
                        'channel-group 1 mode on',
                        'default interface %s' % et3])

            api = dut.api('interfaces')
            result = api.set_members('Port-Channel1', [et1, et3])
            self.assertTrue(result, 'dut=%s' % dut)

            cmd = 'show running-config interfaces %s'

            # check to make sure et1 is still in the lag and et3 was
            # added to the lag
            for interface in [et1, et3]:
                config = dut.run_commands(cmd % interface, 'text')
                self.assertIn('channel-group 1 mode on',
                              config[0]['output'], 'dut=%s' % dut)

            # checks to  make sure et2 was remvoved form the lag
            config = dut.run_commands(cmd % et2, 'text')
            self.assertNotIn('channel-group 1 mode on',
                             config[0]['output'], 'dut=%s' % dut)

    def test_set_members_with_mode(self):
        for dut in self.duts:
            et1 = random_interface(dut)
            et2 = random_interface(dut, exclude=[et1])
            et3 = random_interface(dut, exclude=[et1, et2])

            dut.config(['no interface Port-Channel1',
                        'default interface %s' % et1,
                        'interface %s' % et1,
                        'channel-group 1 mode on',
                        'default interface %s' % et2,
                        'interface %s' % et2,
                        'channel-group 1 mode on',
                        'default interface %s' % et3])

            api = dut.api('interfaces')
            result = api.set_members('Port-Channel1', [et1, et3], mode='active')
            self.assertTrue(result, 'dut=%s' % dut)

            cmd = 'show running-config interfaces %s'

            # check to make sure et1 is still in the lag and et3 was
            # added to the lag
            for interface in [et1, et3]:
                config = dut.run_commands(cmd % interface, 'text')
                self.assertIn('channel-group 1 mode active',
                              config[0]['output'], 'dut=%s' % dut)

            # checks to  make sure et2 was remvoved form the lag
            config = dut.run_commands(cmd % et2, 'text')
            self.assertNotIn('channel-group 1 mode on',
                             config[0]['output'], 'dut=%s' % dut)

    def test_get_members_default(self):
        for dut in self.duts:
            dut.config(['no interface Port-Channel1',
                        'interface Port-Channel1'])
            instance = dut.api('interfaces').get_instance('Port-Channel1')
            result = instance.get_members('Port-Channel1')
            self.assertEqual(result, list(), 'dut=%s' % dut)

    def test_get_members_one_member(self):
        for dut in self.duts:
            dut.config(['no interface Port-Channel1',
                        'interface Port-Channel1',
                        'default interface Ethernet1',
                        'interface Ethernet1',
                        'channel-group 1 mode active'])
            instance = dut.api('interfaces').get_instance('Port-Channel1')
            result = instance.get_members('Port-Channel1')
            self.assertEqual(result, ['Ethernet1'], 'dut=%s' % dut)

    def test_get_members_two_members(self):
        for dut in self.duts:
            dut.config(['no interface Port-Channel1',
                        'interface Port-Channel1',
                        'default interface Ethernet1-2',
                        'interface Ethernet1-2',
                        'channel-group 1 mode active'])
            instance = dut.api('interfaces').get_instance('Port-Channel1')
            result = instance.get_members('Port-Channel1')
            self.assertEqual(result, ['Ethernet1', 'Ethernet2'], 'dut=%s' % dut)

    def test_set_lacp_mode(self):
        for dut in self.duts:
            for mode in ['on', 'active', 'passive']:
                cfgmode = 'on' if mode != 'on' else 'active'
                dut.config(['no interface Port-Channel1',
                            'default interface Ethernet1',
                            'interface Ethernet1',
                            'channel-group 1 mode %s' % cfgmode])

                result = dut.api('interfaces').set_lacp_mode('Port-Channel1',
                                                             mode)
                self.assertTrue(result, 'dut=%s' % dut)
                commands = 'show running-config interfaces Ethernet1'
                config = dut.run_commands(commands, 'text')
                self.assertIn('channel-group 1 mode %s' % mode,
                              config[0]['output'], 'dut=%s' % dut)

    def test_set_lacp_mode_invalid_value(self):
        for dut in self.duts:
            mode = random_string()
            result = dut.api('interfaces').set_lacp_mode('Port-Channel1', mode)
            self.assertFalse(result)

    def test_get_lacp_mode_with_default(self):
        for dut in self.duts:
            dut.config(['no interface Port-Channel1',
                        'interface Port-Channel1'])
            instance = dut.api('interfaces').get_instance('Port-Channel1')
            result = instance.get_lacp_mode('Port-Channel1')
            self.assertEqual(result, 'on', 'dut=%s' % dut)

    def test_minimum_links_valid(self):
        for dut in self.duts:
            minlinks = random_int(1, 16)
            dut.config(['no interface Port-Channel1',
                        'interface Port-Channel1'])
            result = dut.api('interfaces').set_minimum_links('Port-Channel1',
                                                             minlinks)
            self.assertTrue(result, 'dut=%s' % dut)
            commands = 'show running-config interfaces Port-Channel1'
            config = dut.run_commands(commands, 'text')
            self.assertIn('port-channel min-links %s' % minlinks,
                          config[0]['output'], 'dut=%s' % dut)

    def test_minimum_links_invalid_value(self):
        for dut in self.duts:
            minlinks = random_int(17, 128)
            result = dut.api('interfaces').set_minimum_links('Port-Channel1',
                                                             minlinks)
            self.assertFalse(result)

    def test_create_and_delete_portchannel_sub_interface(self):
        for dut in self.duts:
            et1 = random_interface(dut)
            et2 = random_interface(dut, exclude=[et1])

            dut.config(['no interface Port-Channel1',
                        'default interface %s' % et1,
                        'interface %s' % et1,
                        'channel-group 1 mode on',
                        'default interface %s' % et2,
                        'interface %s' % et2,
                        'channel-group 1 mode on'])
            # Create subint Port-Channel1.1
            api = dut.api('interfaces')
            result = api.create('Port-Channel1.1')
            self.assertTrue(result, 'dut=%s' % dut)
            command = 'show running-config interfaces Port-Channel1.1'
            output = dut.run_commands(command, encoding='text')
            self.assertIn('Port-Channel1.1', output[0]['output'])
            # Delete subint Port-Channel1.1
            result = dut.api('interfaces').delete('Port-Channel1.1')
            self.assertTrue(result)
            output = dut.run_commands(command, encoding='text')
            self.assertEqual(output[0]['output'], '')
            # Remove port-channel and default interfaces
            dut.config(['no interface Port-Channel1',
                        'default interface %s' % et1,
                        'default interface %s' % et2])

    def test_set_and_unset_portchannel_sub_intf_encapsulation(self):
        for dut in self.duts:
            et1 = random_interface(dut)
            et2 = random_interface(dut, exclude=[et1])

            dut.config(['no interface Port-Channel1',
                        'default interface %s' % et1,
                        'interface %s' % et1,
                        'channel-group 1 mode on',
                        'default interface %s' % et2,
                        'interface %s' % et2,
                        'channel-group 1 mode on'])
            # Create subint Port-Channel1.1
            api = dut.api('interfaces')
            result = api.create('Port-Channel1.1')
            self.assertTrue(result)
            # Set encapsulation
            result = api.set_encapsulation('Port-Channel1.1', 4)
            self.assertTrue(result)
            command = 'show running-config interfaces Port-Channel1.1'
            output = dut.run_commands(command, encoding='text')
            encap = 'encapsulation dot1q vlan 4'
            self.assertIn(encap, output[0]['output'])
            # Unset encapsulation
            result = api.set_encapsulation('Port-Channel1.1', 4, default=True)
            self.assertTrue(result)
            output = dut.run_commands(command, encoding='text')
            self.assertNotIn(encap, output[0]['output'])
            # Delete subint Port-Channel1.1
            result = dut.api('interfaces').delete('Port-Channel1.1')
            self.assertTrue(result)
            output = dut.run_commands(command, encoding='text')
            self.assertEqual(output[0]['output'], '')
            # Remove port-channel and default interfaces
            dut.config(['no interface Port-Channel1',
                        'default interface %s' % et1,
                        'default interface %s' % et2])


class TestApiVxlanInterface(DutSystemTest):

    def test_get(self):
        for dut in self.duts:
            dut.config(['no interface Vxlan1',
                        'interface Vxlan1'])
            result = dut.api('interfaces').get('Vxlan1')
            self.assertIsInstance(result, dict)
            self.assertEqual(result['type'], 'vxlan')
            self.assertEqual(result['name'], 'Vxlan1')
            self.assertFalse(result['shutdown'])
            self.assertEqual(result['description'], None)
            self.assertEqual(result['source_interface'], '')
            self.assertEqual(result['multicast_group'], '')
            self.assertEqual(result['multicast_decap'], False)


    def get_config(self, dut):
        cmd = 'show running-config all interfaces Vxlan1'
        config = dut.run_commands(cmd, 'text')
        return config[0]['output']

    def contains(self, text, dut):
        self.assertIn(text, self.get_config(dut), 'dut=%s' % dut)

    def notcontains(self, text, dut):
        self.assertNotIn(text, self.get_config(dut), 'dut=%s' % dut)

    def test_set_source_interface(self):
        for dut in self.duts:
            dut.config(['no interface Vxlan1', 'interface Vxlan1'])
            api = dut.api('interfaces')
            instance = api.set_source_interface('Vxlan1', 'Loopback0')
            self.assertTrue(instance)
            self.contains('vxlan source-interface Loopback0', dut)

    def test_set_source_interface_default(self):
        for dut in self.duts:
            dut.config(['no interface Vxlan1', 'interface Vxlan1',
                        'vxlan source-interface Loopback0'])
            api = dut.api('interfaces')
            instance = api.set_source_interface('Vxlan1', default=True)
            self.assertTrue(instance)
            self.contains('no vxlan source-interface', dut)

    def test_set_source_interface_negate(self):
        for dut in self.duts:
            dut.config(['no interface Vxlan1', 'interface Vxlan1',
                        'vxlan source-interface Loopback0'])
            api = dut.api('interfaces')
            instance = api.set_source_interface('Vxlan1', disable=True)
            self.assertTrue(instance)
            self.contains('no vxlan source-interface', dut)

    def test_set_multicast_group(self):
        for dut in self.duts:
            dut.config(['no interface Vxlan1', 'interface Vxlan1'])
            api = dut.api('interfaces')
            instance = api.set_multicast_group('Vxlan1', '239.10.10.10')
            self.assertTrue(instance)
            self.contains('vxlan multicast-group 239.10.10.10', dut)

    def test_set_multicast_group_default(self):
        for dut in self.duts:
            dut.config(['no interface Vxlan1', 'interface Vxlan1',
                        'vxlan multicast-group 239.10.10.10'])
            api = dut.api('interfaces')
            instance = api.set_multicast_group('Vxlan1', default=True)
            self.assertTrue(instance)
            self.contains('no vxlan multicast-group', dut)

    def test_set_multicast_group_negate(self):
        for dut in self.duts:
            dut.config(['no interface Vxlan1', 'interface Vxlan1',
                        'vxlan multicast-group 239.10.10.10'])
            api = dut.api('interfaces')
            instance = api.set_multicast_group('Vxlan1', disable=True)
            self.assertTrue(instance)
            self.contains('no vxlan multicast-group', dut)

    '''commenting this one out as it will only parse on a  trident based DUT
    def test_set_multicast_decap(self):
        for dut in self.duts:
            dut.config(['no interface Vxlan1', 'interface Vxlan1'])
            api = dut.api('interfaces')
            instance = api.set_multicast_decap('Vxlan1')
            self.assertTrue(instance)
            self.contains('vxlan multicast-group decap', dut)
    '''

    def test_set_udp_port(self):
        for dut in self.duts:
            dut.config(['no interface Vxlan1', 'interface Vxlan1',
                        'vxlan udp-port 1024'])
            api = dut.api('interfaces')
            instance = api.set_udp_port('Vxlan1', '1024')
            self.assertTrue(instance)
            self.contains('vxlan udp-port 1024', dut)

    def test_set_udp_port_default(self):
        for dut in self.duts:
            dut.config(['no interface Vxlan1', 'interface Vxlan1',
                        'vxlan udp-port 1024'])
            api = dut.api('interfaces')
            instance = api.set_udp_port('Vxlan1', default=True)
            self.assertTrue(instance)
            self.contains('vxlan udp-port 4789', dut)

    def test_set_udp_port_negate(self):
        for dut in self.duts:
            dut.config(['no interface Vxlan1', 'interface Vxlan1',
                        'vxlan udp-port 1024'])
            api = dut.api('interfaces')
            instance = api.set_udp_port('Vxlan1', disable=True)
            self.assertTrue(instance)
            self.contains('vxlan udp-port 4789', dut)

    def test_add_vtep(self):
        for dut in self.duts:
            dut.config(['no interface Vxlan1', 'interface Vxlan1'])
            api = dut.api('interfaces')
            instance = api.add_vtep('Vxlan1', '1.1.1.1')
            self.assertTrue(instance)
            self.contains('vxlan flood vtep 1.1.1.1', dut)

    def test_add_vtep_to_vlan(self):
        for dut in self.duts:
            dut.config(['no interface Vxlan1', 'interface Vxlan1'])
            api = dut.api('interfaces')
            instance = api.add_vtep('Vxlan1', '1.1.1.1', vlan='10')
            self.assertTrue(instance)
            self.contains('vxlan vlan 10 flood vtep 1.1.1.1', dut)

    def test_remove_vtep(self):
        for dut in self.duts:
            dut.config(['no interface Vxlan1', 'interface Vxlan1'])
            api = dut.api('interfaces')
            instance = api.remove_vtep('Vxlan1', '1.1.1.1')
            self.assertTrue(instance)
            self.contains('no vxlan flood vtep', dut)

    def test_remove_vtep_from_vlan(self):
        for dut in self.duts:
            dut.config(['no interface Vxlan1', 'interface Vxlan1'])
            api = dut.api('interfaces')
            instance = api.remove_vtep('Vxlan1', '1.1.1.1', vlan='10')
            self.assertTrue(instance)
            self.notcontains('vxlan vlan 10 flood vtep remove 1.1.1.1', dut)

    def test_update_vlan(self):
        for dut in self.duts:
            dut.config(['no interface Vxlan1', 'interface Vxlan1'])
            api = dut.api('interfaces')
            instance = api.update_vlan('Vxlan1', '10', '10')
            self.assertTrue(instance)
            self.contains('vxlan vlan add 10 vni 10', dut)

    def test_remove_vlan(self):
        for dut in self.duts:
            dut.config(['no interface Vxlan1', 'interface Vxlan1'])
            api = dut.api('interfaces')
            instance = api.remove_vlan('Vxlan1', '10')
            self.assertTrue(instance)
            self.notcontains('vxlan vlan remove 10 vni 10', dut)


if __name__ == '__main__':
    unittest.main()



#
# Copyright (c) 2015, Arista Networks, Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
#   Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
#   Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
#
#   Neither the name of Arista Networks nor the names of its
#   contributors may be used to endorse or promote products derived from
#   this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL ARISTA NETWORKS
# BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
# BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
# WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
# OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
# IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
import sys
import os
import unittest

sys.path.append(os.path.join(os.path.dirname(__file__), '../lib'))

from systestlib import DutSystemTest

IP_PREFIX = '10.10.10.'
VR_CONFIG = {
    'primary_ip': '10.10.10.2',
    'priority': 200,
    'description': 'modified vrrp 10 on an interface',
    'secondary_ip': ['10.10.10.11'],
    'ip_version': 3,
    'enable': False,
    'timers_advertise': 2,
    'mac_addr_adv_interval': 3,
    'preempt': False,
    'preempt_delay_min': 1,
    'preempt_delay_reload': None,
    'delay_reload': 1,
    'track': [
        {'name': 'Ethernet1', 'action': 'shutdown'},
        {'name': 'Ethernet2', 'action': 'decrement', 'amount': 10},
        {'name': 'Ethernet2', 'action': 'shutdown'},
    ],
    'bfd_ip': '10.10.10.150',
}


class TestApiVrrp(DutSystemTest):

    def _vlan_setup(self, dut):
        dut.config(['no interface vlan 101',
                    'interface vlan 101',
                    'ip address %s1/24' % IP_PREFIX,
                    'exit'])
        return 'Vlan101'

    def test_get(self):
        vrid = 98
        for dut in self.duts:
            interface = self._vlan_setup(dut)
            if dut.version_number >= '4.23':
                dut.config(['interface %s' % interface,
                            'vrrp %d disabled' % vrid,
                            'exit'])
            else:
                dut.config(['interface %s' % interface,
                            'vrrp %d shutdown' % vrid,
                            'exit'])
            response = dut.api('vrrp').get(interface)

            self.assertIsNotNone(response)

    def test_getall(self):
        vrid = 98
        vrid2 = 198
        for dut in self.duts:
            interface = self._vlan_setup(dut)
            if dut.version_number >= '4.23':
                dut.config(['interface %s' % interface,
                            'vrrp %d disabled' % vrid,
                            'exit',
                            'interface vlan $',
                            'vrrp %d disabled' % vrid,
                            'vrrp %d disabled' % vrid2,
                            'exit'])
            else:
                dut.config(['interface %s' % interface,
                            'vrrp %d shutdown' % vrid,
                            'exit',
                            'interface vlan $',
                            'vrrp %d shutdown' % vrid,
                            'vrrp %d shutdown' % vrid2,
                            'exit'])
            response = dut.api('vrrp').getall()

            self.assertIsNotNone(response)

    def test_create(self):
        vrid = 99
        import copy
        vrrp_conf = copy.deepcopy(VR_CONFIG)
        # vrrp_conf = dict(VR_CONFIG)
        for dut in self.duts:
            interface = self._vlan_setup(dut)
            dut.config(['interface %s' % interface,
                        'no vrrp %d' % vrid,
                        'exit'])

            response = dut.api('vrrp').create(interface, vrid, **vrrp_conf)
            self.assertIs(response, True)

            # Fix the configuration dict for proper output
            vrrp_conf = dut.api('vrrp').vrconf_format(vrrp_conf)

            response = dut.api('vrrp').get(interface)[vrid]

            self.maxDiff = None
            self.assertEqual(response, vrrp_conf)

    def test_delete(self):
        vrid = 101
        for dut in self.duts:
            interface = self._vlan_setup(dut)
            if dut.version_number >= '4.23':
                dut.config(['interface %s' % interface,
                            'no vrrp %d' % vrid,
                            'vrrp %d disabled' % vrid,
                            'exit'])
            else:
                dut.config(['interface %s' % interface,
                            'no vrrp %d' % vrid,
                            'vrrp %d shutdown' % vrid,
                            'exit'])

            response = dut.api('vrrp').delete(interface, vrid)
            self.assertIs(response, True)

    def test_default(self):
        vrid = 102
        for dut in self.duts:
            interface = self._vlan_setup(dut)
            if dut.version_number >= '4.23':
                dut.config(['interface %s' % interface,
                            'no vrrp %d' % vrid,
                            'vrrp %d disabled' % vrid,
                            'exit'])
            else:
                dut.config(['interface %s' % interface,
                            'no vrrp %d' % vrid,
                            'vrrp %d shutdown' % vrid,
                            'exit'])

            response = dut.api('vrrp').delete(interface, vrid)
            self.assertIs(response, True)

    def test_update_with_create(self):
        pass
        vrid = 103
        import copy
        vrrp_conf = copy.deepcopy(VR_CONFIG)
        # vrrp_conf = dict(VR_CONFIG)
        for dut in self.duts:
            interface = self._vlan_setup(dut)
            dut.config(['interface %s' % interface,
                        'no vrrp %d' % vrid,
                        'exit'])

            # Create the inital vrrp on the interface
            response = dut.api('vrrp').create(interface, vrid, **vrrp_conf)
            self.assertIs(response, True)

            # Update some of the information on the vrrp
            vrrp_update = {
                'primary_ip': '10.10.10.12',
                'priority': 200,
                'description': 'updated vrrp 10 on an interface',
                'secondary_ip': ['10.10.10.13', '10.10.10.23'],
                'ip_version': 2,
                'enable': True,
                'timers_advertise': None,
                'mac_addr_adv_interval': 'default',
                'preempt': True,
                'preempt_delay_min': 'default',
                'preempt_delay_reload': 'default',
                'delay_reload': 'default',
                'track': [
                    {'name': 'Ethernet2', 'action': 'shutdown'},
                    {'name': 'Ethernet2', 'action': 'decrement', 'amount': 1},
                ],
                'bfd_ip': None,
            }

            response = dut.api('vrrp').create(interface, vrid, **vrrp_update)
            self.assertIs(response, True)
            vrrp_update = dut.api('vrrp').vrconf_format(vrrp_update)

            response = dut.api('vrrp').get(interface)[vrid]

            self.maxDiff = None
            self.assertEqual(response, vrrp_update)

    def test_set_enable(self):
        vrid = 104
        enable_cases = [
            {'value': True},
            {'value': False},
            {'value': True},
            {'value': False},
        ]
        for dut in self.duts:
            interface = self._vlan_setup(dut)
            if dut.version_number >= '4.23':
                dut.config(['interface %s' % interface,
                            'no vrrp %d' % vrid,
                            'vrrp %d disabled' % vrid,
                            'vrrp %d ipv4 10.10.10.2' % vrid,
                            'exit'])
            else:
                dut.config(['interface %s' % interface,
                            'no vrrp %d' % vrid,
                            'vrrp %d shutdown' % vrid,
                            'vrrp %d ip 10.10.10.2' % vrid,
                            'exit'])

            for enable in enable_cases:
                response = dut.api('vrrp').set_enable(
                    interface, vrid, **enable)
                self.assertIs(response, True)

    def test_set_primary_ip(self):
        vrid = 104
        primary_ip_cases = [
            {'value': '10.10.10.2'},
            {'default': True},
            {'value': '10.10.10.3'},
            {'disable': True},
            {'value': '10.10.10.4'},
        ]
        for dut in self.duts:
            interface = self._vlan_setup(dut)
            if dut.version_number >= '4.23':
                dut.config(['interface %s' % interface,
                            'no vrrp %d' % vrid,
                            'vrrp %d disabled' % vrid,
                            'exit'])
            else:
                dut.config(['interface %s' % interface,
                            'no vrrp %d' % vrid,
                            'vrrp %d shutdown' % vrid,
                            'exit'])

            for p_ip in primary_ip_cases:
                response = dut.api('vrrp').set_primary_ip(
                    interface, vrid, **p_ip)
                self.assertIs(response, True)

    def test_set_priority(self):
        vrid = 104
        priority_cases = [
            {'value': 200},
            {'default': True},
            {'value': 175},
            {'disable': True},
            {'value': 190}
        ]
        for dut in self.duts:
            interface = self._vlan_setup(dut)
            if dut.version_number >= '4.23':
                dut.config(['interface %s' % interface,
                            'no vrrp %d' % vrid,
                            'vrrp %d disabled' % vrid,
                            'exit'])
            else:
                dut.config(['interface %s' % interface,
                            'no vrrp %d' % vrid,
                            'vrrp %d shutdown' % vrid,
                            'exit'])

            for priority in priority_cases:
                response = dut.api('vrrp').set_priority(
                    interface, vrid, **priority)
                self.assertIs(response, True)

    def test_set_description(self):
        vrid = 104
        desc_cases = [
            {'value': '1st modified vrrp'},
            {'default': True},
            {'value': '2nd modified vrrp'},
            {'disable': True},
            {'value': '3rd modified vrrp'},
        ]
        for dut in self.duts:
            interface = self._vlan_setup(dut)
            if dut.version_number >= '4.23':
                dut.config(['interface %s' % interface,
                            'no vrrp %d' % vrid,
                            'vrrp %d disabled' % vrid,
                            'exit'])
            else:
                dut.config(['interface %s' % interface,
                            'no vrrp %d' % vrid,
                            'vrrp %d shutdown' % vrid,
                            'exit'])

            for description in desc_cases:
                response = dut.api('vrrp').set_description(
                    interface, vrid, **description)
                self.assertIs(response, True)

    def test_set_secondary_ips(self):
        vrid = 104
        secondary_ip_cases = [
            ['10.10.10.51', '10.10.10.52'],
            ['10.10.10.53', '10.10.10.54'],
            [],
        ]
        for dut in self.duts:
            interface = self._vlan_setup(dut)
            if dut.version_number >= '4.23':
                dut.config(['interface %s' % interface,
                            'no vrrp %d' % vrid,
                            'vrrp %d disabled' % vrid,
                            'exit'])
            else:
                dut.config(['interface %s' % interface,
                            'no vrrp %d' % vrid,
                            'vrrp %d shutdown' % vrid,
                            'exit'])

            for s_ip_list in secondary_ip_cases:
                response = dut.api('vrrp').set_secondary_ips(
                    interface, vrid, s_ip_list)
                self.assertIs(response, True)

    def test_set_ip_version(self):
        vrid = 104
        ip_version_cases = [
            {'value': 2},
            {'value': 3},
            {'default': True},
            {'value': 3},
            {'disable': True},
            {'value': 3},
        ]
        for dut in self.duts:
            interface = self._vlan_setup(dut)
            if dut.version_number >= '4.23':
                dut.config(['interface %s' % interface,
                            'no vrrp %d' % vrid,
                            'vrrp %d disabled' % vrid,
                            'exit'])
            else:
                dut.config(['interface %s' % interface,
                            'no vrrp %d' % vrid,
                            'vrrp %d shutdown' % vrid,
                            'exit'])

            for ip_version in ip_version_cases:
                response = dut.api('vrrp').set_ip_version(
                    interface, vrid, **ip_version)
                self.assertIs(response, True)

    def test_set_timers_advertise(self):
        vrid = 104
        timers_adv_cases = [
            {'value': 10},
            {'default': True},
            {'value': 20},
            {'disable': True},
            {'value': 30},
        ]
        for dut in self.duts:
            interface = self._vlan_setup(dut)
            if dut.version_number >= '4.23':
                dut.config(['interface %s' % interface,
                            'no vrrp %d' % vrid,
                            'vrrp %d disabled' % vrid,
                            'exit'])
            else:
                dut.config(['interface %s' % interface,
                            'no vrrp %d' % vrid,
                            'vrrp %d shutdown' % vrid,
                            'exit'])

            for timers_advertise in timers_adv_cases:
                response = dut.api('vrrp').set_timers_advertise(
                    interface, vrid, **timers_advertise)
                self.assertIs(response, True)

    def test_set_mac_addr_adv_interval(self):
        vrid = 104
        mac_addr_adv_int_cases = [
            {'value': 50},
            {'default': True},
            {'value': 55},
            {'disable': True},
            {'value': 60},
        ]
        for dut in self.duts:
            interface = self._vlan_setup(dut)
            if dut.version_number >= '4.23':
                dut.config(['interface %s' % interface,
                            'no vrrp %d' % vrid,
                            'vrrp %d disabled' % vrid,
                            'exit'])
            else:
                dut.config(['interface %s' % interface,
                            'no vrrp %d' % vrid,
                            'vrrp %d shutdown' % vrid,
                            'exit'])

            for mac_addr_adv_intvl in mac_addr_adv_int_cases:
                response = dut.api('vrrp').set_mac_addr_adv_interval(
                    interface, vrid, **mac_addr_adv_intvl)
                self.assertIs(response, True)

    def test_set_preempt(self):
        vrid = 104
        preempt_cases = [
            {'value': True},
            {'default': True},
            {'value': True},
            {'disable': True},
            {'value': True},
            {'value': False},
        ]
        for dut in self.duts:
            interface = self._vlan_setup(dut)
            if dut.version_number >= '4.23':
                dut.config(['interface %s' % interface,
                            'no vrrp %d' % vrid,
                            'vrrp %d disabled' % vrid,
                            'exit'])
            else:
                dut.config(['interface %s' % interface,
                            'no vrrp %d' % vrid,
                            'vrrp %d shutdown' % vrid,
                            'exit'])

            for preempt in preempt_cases:
                response = dut.api('vrrp').set_preempt(
                    interface, vrid, **preempt)
                self.assertIs(response, True)

    def test_set_preempt_delay_min(self):
        vrid = 104
        preempt_delay_min_cases = [
            {'value': 3600},
            {'default': True},
            {'value': 500},
            {'disable': True},
            {'value': 150},
        ]
        for dut in self.duts:
            interface = self._vlan_setup(dut)
            if dut.version_number >= '4.23':
                dut.config(['interface %s' % interface,
                            'no vrrp %d' % vrid,
                            'vrrp %d disabled' % vrid,
                            'exit'])
            else:
                dut.config(['interface %s' % interface,
                            'no vrrp %d' % vrid,
                            'vrrp %d shutdown' % vrid,
                            'exit'])

            for preempt_delay_min in preempt_delay_min_cases:
                response = dut.api('vrrp').set_preempt_delay_min(
                    interface, vrid, **preempt_delay_min)
                self.assertIs(response, True)

    def test_set_preempt_delay_reload(self):
        vrid = 104
        preempt_delay_reload_cases = [
            {'value': 3600},
            {'default': True},
            {'value': 500},
            {'disable': True},
            {'value': 150},
        ]
        for dut in self.duts:
            interface = self._vlan_setup(dut)
            if dut.version_number >= '4.23':
                dut.config(['interface %s' % interface,
                            'no vrrp %d' % vrid,
                            'vrrp %d disabled' % vrid,
                            'exit'])
            else:
                dut.config(['interface %s' % interface,
                            'no vrrp %d' % vrid,
                            'vrrp %d shutdown' % vrid,
                            'exit'])

            for preempt_delay_reload in preempt_delay_reload_cases:
                response = dut.api('vrrp').set_preempt_delay_reload(
                    interface, vrid, **preempt_delay_reload)
                self.assertIs(response, True)

    def test_set_delay_reload(self):
        vrid = 104
        delay_reload_cases = [
            {'value': 30},
            {'default': True},
            {'value': 25},
            {'disable': True},
            {'value': 15},
        ]
        for dut in self.duts:
            interface = self._vlan_setup(dut)
            if dut.version_number >= '4.23':
                dut.config(['interface %s' % interface,
                            'no vrrp %d' % vrid,
                            'vrrp %d disabled' % vrid,
                            'exit'])
            else:
                dut.config(['interface %s' % interface,
                            'no vrrp %d' % vrid,
                            'vrrp %d shutdown' % vrid,
                            'exit'])

            for delay_reload in delay_reload_cases:
                response = dut.api('vrrp').set_delay_reload(
                    interface, vrid, **delay_reload)
                self.assertIs(response, True)

    def test_set_tracks(self):
        vrid = 104
        track_cases = [
            [
                {'name': 'Ethernet1', 'action': 'shutdown'},
                {'name': 'Ethernet2', 'action': 'decrement', 'amount': 10},
                {'name': 'Ethernet2', 'action': 'shutdown'},
            ],
            [
                {'name': 'Ethernet1', 'action': 'shutdown'},
            ],
            [
                {'name': 'Ethernet1', 'action': 'shutdown'},
                {'name': 'Ethernet2', 'action': 'decrement', 'amount': 20},
                {'name': 'Ethernet2', 'action': 'shutdown'},
            ],
            [
                {'name': 'Ethernet1', 'action': 'shutdown'},
            ],
            [],
        ]
        for dut in self.duts:
            interface = self._vlan_setup(dut)
            if dut.version_number >= '4.23':
                dut.config(['interface %s' % interface,
                            'no vrrp %d' % vrid,
                            'vrrp %d disabled' % vrid,
                            'exit'])
            else:
                dut.config(['interface %s' % interface,
                            'no vrrp %d' % vrid,
                            'vrrp %d shutdown' % vrid,
                            'exit'])

            for track_list in track_cases:
                response = dut.api('vrrp').set_tracks(
                    interface, vrid, track_list)
                self.assertIs(response, True)

    def test_set_bfd_ip(self):
        vrid = 104
        bfd_ip_cases = [
            {'value': '10.10.10.160'},
            {'default': True},
            {'value': '10.10.10.161'},
            {'disable': True},
            {'value': '10.10.10.162'},
        ]
        for dut in self.duts:
            interface = self._vlan_setup(dut)
            if dut.version_number >= '4.23':
                dut.config(['interface %s' % interface,
                            'no vrrp %d' % vrid,
                            'vrrp %d disabled' % vrid,
                            'exit'])
            else:
                dut.config(['interface %s' % interface,
                            'no vrrp %d' % vrid,
                            'vrrp %d shutdown' % vrid,
                            'exit'])

            for bfd_ip in bfd_ip_cases:
                response = dut.api('vrrp').set_bfd_ip(
                    interface, vrid, **bfd_ip)
                self.assertIs(response, True)


if __name__ == '__main__':
    unittest.main()



#
# Copyright (c) 2015, Arista Networks, Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
#   Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
#   Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
#
#   Neither the name of Arista Networks nor the names of its
#   contributors may be used to endorse or promote products derived from
#   this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL ARISTA NETWORKS
# BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
# BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
# WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
# OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
# IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
import sys
import os
import unittest

sys.path.append(os.path.join(os.path.dirname(__file__), '../lib'))

from testlib import get_fixture, function, random_string
from testlib import EapiConfigUnitTest

import pyeapi.api.routemaps


class TestApiRoutemaps(EapiConfigUnitTest):

    def __init__(self, *args, **kwargs):
        super(TestApiRoutemaps, self).__init__(*args, **kwargs)
        self.instance = pyeapi.api.routemaps.Routemaps(None)
        self.config = open(get_fixture('running_config.routemaps')).read()

    def test_instance(self):
        result = pyeapi.api.routemaps.instance(None)
        self.assertIsInstance(result, pyeapi.api.routemaps.Routemaps)

    def test_get(self):
        result = self.instance.get('TEST')
        keys = ['deny', 'permit']
        self.assertEqual(sorted(keys), sorted(result.keys()))

    def test_get_not_configured(self):
        self.assertIsNone(self.instance.get('blah'))

    def test_getall(self):
        # Review fixtures/running_config.routemaps to see the default
        # running-config that is the basis for this test
        result = self.instance.getall()
        self.assertIsInstance(result, dict)
        self.assertEqual(len(result.keys()), 4)

    def test_routemaps_functions(self):
        for name in ['create', 'delete', 'default']:
            if name == 'create':
                cmds = 'route-map new permit 100'
            elif name == 'delete':
                cmds = 'no route-map new permit 100'
            elif name == 'default':
                cmds = 'default route-map new permit 100'
            func = function(name, 'new', 'permit', 100)
            self.eapi_positive_config_test(func, cmds)

    def test_set_set_statement_clean(self):
        cmds = ['route-map new permit 100', 'set weight 100']
        func = function('set_set_statements', 'new', 'permit', 100,
                        ['weight 100'])
        self.eapi_positive_config_test(func, cmds)

    def test_set_set_statement_remove_extraneous(self):
        # Review fixtures/running_config.routemaps to see the default
        # running-config that is the basis for this test
        cmds = ['route-map TEST permit 10', 'no set tag 50',
                'route-map TEST permit 10', 'set weight 100']
        func = function('set_set_statements', 'TEST', 'permit', 10,
                        ['weight 100'])
        self.eapi_positive_config_test(func, cmds)

    def test_set_match_statement_clean(self):
        cmds = ['route-map new permit 200', 'match as 100']
        func = function('set_match_statements', 'new', 'permit', 200,
                        ['as 100'])
        self.eapi_positive_config_test(func, cmds)

    def test_set_match_statement_remove_extraneous(self):
        # Review fixtures/running_config.routemaps to see the default
        # running-config that is the basis for this test
        cmds = ['route-map TEST permit 10', 'no match interface Ethernet1',
                'route-map TEST permit 10', 'match as 1000']
        func = function('set_match_statements', 'TEST', 'permit', 10,
                        ['as 1000'])
        self.eapi_positive_config_test(func, cmds)

    def test_set_continue(self):
        cmds = ['route-map TEST permit 10', 'continue 100']
        func = function('set_continue', 'TEST', 'permit', 10, 100)
        self.eapi_positive_config_test(func, cmds)

    def test_set_continue_with_invalid_integer(self):
        with self.assertRaises(ValueError):
            self.instance.set_continue('TEST', 'permit', 10, -1)

    def test_set_continue_with_invalid_string(self):
        with self.assertRaises(ValueError):
            self.instance.set_continue('TEST', 'permit', 10, 'invalid')

    def test_set_continue_to_default(self):
        cmds = ['route-map TEST permit 10', 'default continue']
        func = function('set_continue', 'TEST', 'permit', 10, default=True)
        self.eapi_positive_config_test(func, cmds)

    def test_negate_continue(self):
        cmds = ['route-map TEST permit 10', 'no continue']
        func = function('set_continue', 'TEST', 'permit', 10, disable=True)
        self.eapi_positive_config_test(func, cmds)

    def test_set_description_with_value(self):
        value = random_string()
        cmds = ['route-map TEST permit 10', 'no description',
                'description %s' % value]
        func = function('set_description', 'TEST', 'permit', 10, value)
        self.eapi_positive_config_test(func, cmds)

    def test_negate_description(self):
        cmds = ['route-map TEST permit 10', 'no description']
        func = function('set_description', 'TEST', 'permit', 10, disable=True)
        self.eapi_positive_config_test(func, cmds)

    def test_set_description_with_default(self):
        cmds = ['route-map TEST permit 10', 'default description']
        func = function('set_description', 'TEST', 'permit', 10, default=True)
        self.eapi_positive_config_test(func, cmds)

    def test_set_description_with_invalid_value(self):
        # with self.assertRaises(ValueError):
        #     self.instance.set_description('TEST', 'permit', 10, value=None)
        # If command_builder fails because value is None, uncomment
        # above lines and remove below lines.
        cmds = ['route-map TEST permit 10', 'no description']
        func = function('set_description', 'TEST', 'permit', 10, value=None)
        self.eapi_positive_config_test(func, cmds)


if __name__ == '__main__':
    unittest.main()



#
# Copyright (c) 2014, Arista Networks, Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
#   Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
#   Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
#
#   Neither the name of Arista Networks nor the names of its
#   contributors may be used to endorse or promote products derived from
#   this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL ARISTA NETWORKS
# BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
# BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
# WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
# OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
# IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
import sys
import os
import unittest

sys.path.append(os.path.join(os.path.dirname(__file__), '../lib'))

from random import choice
from testlib import random_int, random_string
from systestlib import DutSystemTest

NEXT_HOPS = ['Ethernet1', 'Ethernet2', 'Null0', 'IP']
DISTANCES = TAGS = ROUTE_NAMES = [None, True]


def _ip_addr():
    ip1 = random_int(0, 223)
    ip2 = random_int(0, 255)
    ip3 = random_int(0, 255)
    return "%s.%s.%s.0/24" % (ip1, ip2, ip3)


def _next_hop():
    next_hop = choice(NEXT_HOPS)
    if next_hop == 'Null0':
        return (next_hop, None)
    ip1 = random_int(0, 223)
    ip2 = random_int(0, 255)
    ip3 = random_int(0, 255)
    ip4 = random_int(0, 255)
    ip_addr = "%s.%s.%s.%s" % (ip1, ip2, ip3, ip4)
    if next_hop == 'IP':
        return (ip_addr, None)
    return (next_hop, ip_addr)


def _distance():
    return random_int(1, 255)


def _tag():
    return random_int(0, 255)


def _route_name():
    return random_string(minchar=4, maxchar=10)


class TestApiStaticroute(DutSystemTest):

    def test_create(self):
        # Validate the create function returns without an error
        # when creating routes with varying parameters included.

        for dut in self.duts:
            dut.config(['no ip routing delete-static-routes',
                        'ip routing'])

            for t_distance in DISTANCES:
                for t_tag in TAGS:
                    for t_route_name in ROUTE_NAMES:
                        ip_dest = _ip_addr()
                        (next_hop, next_hop_ip) = _next_hop()
                        distance = t_distance
                        if distance is True:
                            distance = _distance()
                        tag = t_tag
                        if tag is True:
                            tag = _tag()
                        route_name = t_route_name
                        if route_name is True:
                            route_name = _route_name()

                        result = dut.api('staticroute').create(
                            ip_dest, next_hop, next_hop_ip=next_hop_ip,
                            distance=distance, tag=tag, route_name=route_name)

                        self.assertTrue(result)

    def test_get(self):
        # Validate the get function returns the exact value that
        # is passed in when the route exists on the switch.

        for dut in self.duts:
            dut.config(['no ip routing delete-static-routes',
                        'ip routing'])

            ip_dest = '1.2.3.0/24'
            next_hop = 'Ethernet1'
            next_hop_ip = '1.1.1.1'
            distance = 1
            tag = 1
            route_name = 'test1'

            cmd = "ip route %s %s %s %s tag %s name %s" % \
                (ip_dest, next_hop, next_hop_ip, distance, tag, route_name)
            dut.config([cmd])

            route = {
                next_hop: {
                    next_hop_ip: {
                        distance: {
                            'tag': tag,
                            'route_name': route_name
                        }
                    }
                }
            }

            result = dut.api('staticroute').get(ip_dest)

            # Make sure the funtion returns a true result (match found)
            self.assertTrue(result)
            # Then make sure the returned string is what was expected
            # self.assertEqual(result.group(0), cmd)
            self.assertEqual(result, route)

    def test_getall(self):
        # Validate the get_all function returns a list of entries
        # containing the matched parameters, and that parameters
        # are matched in full (i.e. name 'test1' does not match
        # name 'test10').

        for dut in self.duts:
            dut.config(['no ip routing delete-static-routes',
                        'ip routing'])

            # Declare a set of 3 routes with same ip dest and next hop.
            # Set different distance, tag and name for each route,
            # including values 1 and 10 in each, so the test will verify
            # that matching 1 does not also match 10.
            route1 = \
                'ip route 1.2.3.0/24 Ethernet1 1.1.1.1 10 tag 1 name test1'
            route2 = \
                'ip route 1.2.3.0/24 Ethernet1 1.1.1.1 1 tag 1 name test10'
            route3 = \
                'ip route 1.2.3.0/24 Ethernet1 1.1.1.1 2 tag 10 name test1'

            dut.config([route1, route2, route3])

            routes = {
                '1.2.3.0/24': {
                    'Ethernet1': {
                        '1.1.1.1': {
                            10: {
                                'tag': 1,
                                'route_name': 'test1'
                            },
                            1: {
                                'tag': 1,
                                'route_name': 'test10'
                            },
                            2: {
                                'tag': 10,
                                'route_name': 'test1'
                            }
                        }
                    }
                }
            }

            # Get the list of ip routes from the switch
            result = dut.api('staticroute').getall()

            # Assert that the result dict is equivalent to the routes dict
            self.assertEqual(result, routes)

    def test_delete(self):
        # Validate the delete function returns without an error
        # when deleting routes with varying parameters included.
        # Note: the routes do not have to exist for the
        # delete command to succeed, but only that the command
        # does not error.

        for dut in self.duts:
            dut.config(['no ip routing delete-static-routes',
                        'ip routing'])

            for t_distance in DISTANCES:
                for t_tag in TAGS:
                    for t_route_name in ROUTE_NAMES:
                        ip_dest = _ip_addr()
                        (next_hop, next_hop_ip) = _next_hop()
                        distance = t_distance
                        if distance is True:
                            distance = _distance()
                        tag = t_tag
                        if tag is True:
                            tag = _tag()
                        route_name = t_route_name
                        if route_name is True:
                            route_name = _route_name()

                        result = dut.api('staticroute').delete(
                            ip_dest, next_hop, next_hop_ip=next_hop_ip,
                            distance=distance, tag=tag, route_name=route_name)

                        self.assertTrue(result)

    def test_default(self):
        # Validate the default function returns without an error
        # when deleting routes with varying parameters included.
        # Note: currently EOS functionality of 'default ip route ...'
        # is equivalent to 'no ip route ...', which is the delete
        # function.

        for dut in self.duts:
            dut.config(['no ip routing delete-static-routes',
                        'ip routing'])

            for t_distance in DISTANCES:
                for t_tag in TAGS:
                    for t_route_name in ROUTE_NAMES:
                        ip_dest = _ip_addr()
                        (next_hop, next_hop_ip) = _next_hop()
                        distance = t_distance
                        if distance is True:
                            distance = _distance()
                        tag = t_tag
                        if tag is True:
                            tag = _tag()
                        route_name = t_route_name
                        if route_name is True:
                            route_name = _route_name()

                        result = dut.api('staticroute').default(
                            ip_dest, next_hop, next_hop_ip=next_hop_ip,
                            distance=distance, tag=tag, route_name=route_name)

                        self.assertTrue(result)

    def test_set_tag(self):
        # Validate the set_tag function returns without an error
        # when modifying the tag on an existing route

        for dut in self.duts:
            dut.config(['no ip routing delete-static-routes',
                        'ip routing',
                        'ip route 1.2.3.0/24 Ethernet1 1.1.1.1 10 tag 99'])

            result = dut.api('staticroute').set_tag(
                '1.2.3.0/24', 'Ethernet1', next_hop_ip='1.1.1.1',
                distance=10, tag=3)
            self.assertTrue(result)

    def test_set_route_name(self):
        # Validate the set_route_name function returns without an error
        # when modifying the tag on an existing route

        for dut in self.duts:
            dut.config(['no ip routing delete-static-routes',
                        'ip routing',
                        'ip route 1.2.3.0/24 Ethernet1 1.1.1.1 1 name test99'])

            result = dut.api('staticroute').set_route_name(
                '1.2.3.0/24', 'Ethernet1', next_hop_ip='1.1.1.1',
                distance=1, route_name='test3')
            self.assertTrue(result)


if __name__ == '__main__':
    unittest.main()



#
# Copyright (c) 2014, Arista Networks, Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
#   Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
#   Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
#
#   Neither the name of Arista Networks nor the names of its
#   contributors may be used to endorse or promote products derived from
#   this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL ARISTA NETWORKS
# BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
# BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
# WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
# OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
# IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
import sys
import os
import unittest

sys.path.append(os.path.join(os.path.dirname(__file__), '../lib'))

from testlib import get_fixture, random_vlan, random_string, function
from testlib import EapiConfigUnitTest

import pyeapi.api.vlans


class TestApiVlans(EapiConfigUnitTest):

    def __init__(self, *args, **kwargs):
        super(TestApiVlans, self).__init__(*args, **kwargs)
        self.instance = pyeapi.api.vlans.instance(None)
        self.config = open(get_fixture('running_config.text')).read()

    def test_isvlan_with_string(self):
        self.assertFalse(pyeapi.api.vlans.isvlan('a' + random_string()))

    def test_isvlan_valid_value(self):
        self.assertTrue(pyeapi.api.vlans.isvlan('1234'))

    def test_isvlan_invalid_value(self):
        self.assertFalse(pyeapi.api.vlans.isvlan('5000'))

    def test_get(self):
        result = self.instance.get('1')
        vlan = dict(vlan_id='1', name='default', state='active',
                    trunk_groups=[])
        self.assertEqual(vlan, result)

    def test_get_not_configured(self):
        self.assertIsNone(self.instance.get('1000'))

    def test_getall(self):
        result = self.instance.getall()
        self.assertIsInstance(result, dict)
        self.assertEqual(len(result), 4)

    def test_vlan_functions(self):
        for name in ['create', 'delete', 'default']:
            vid = random_vlan()
            if name == 'create':
                cmds = 'vlan %s' % vid
            elif name == 'delete':
                cmds = 'no vlan %s' % vid
            elif name == 'default':
                cmds = 'default vlan %s' % vid
            func = function(name, vid)
            self.eapi_positive_config_test(func, cmds)

    def test_set_name(self):
        for state in ['config', 'negate', 'default']:
            vid = random_vlan()
            name = random_string()
            if state == 'config':
                cmds = ['vlan %s' % vid, 'name %s' % name]
                func = function('set_name', vid, name)
            elif state == 'negate':
                cmds = ['vlan %s' % vid, 'no name']
                func = function('set_name', vid, disable=True)
            elif state == 'default':
                cmds = ['vlan %s' % vid, 'default name']
                func = function('set_name', vid, default=True)
            self.eapi_positive_config_test(func, cmds)

    def test_set_state(self):
        for state in ['config', 'negate', 'default']:
            vid = random_vlan()
            if state == 'config':
                for value in ['active', 'suspend']:
                    cmds = ['vlan %s' % vid, 'state %s' % value]
                    func = function('set_state', vid, value)
                    self.eapi_positive_config_test(func, cmds)
            elif state == 'negate':
                cmds = ['vlan %s' % vid, 'no state']
                func = function('set_state', vid, disable=True)
                self.eapi_positive_config_test(func, cmds)
            elif state == 'default':
                cmds = ['vlan %s' % vid, 'default state']
                func = function('set_state', vid, default=True)
                self.eapi_positive_config_test(func, cmds)

    def test_set_trunk_groups_default(self):
        vid = random_vlan()
        cmds = ['vlan %s' % vid, 'default trunk group']
        func = function('set_trunk_groups', vid, default=True)
        self.eapi_positive_config_test(func, cmds)

    def test_set_trunk_groups_add_value(self):
        cmds = ['vlan 10', 'trunk group tg2']
        func = function('set_trunk_groups', '10', ['tg1', 'tg2'])
        self.eapi_positive_config_test(func, cmds)

    def test_set_trunk_groups_remove_value(self):
        cmds = ['vlan 10', 'no trunk group tg1']
        func = function('set_trunk_groups', '10', 'tg2')
        self.eapi_positive_config_test(func, cmds)

    def test_set_trunk_groups_remove_all(self):
        cmds = ['vlan 10', 'no trunk group']
        func = function('set_trunk_groups', '10', disable=True)
        self.eapi_positive_config_test(func, cmds)

    def test_add_trunk_group(self):
        vid = random_vlan()
        tg = random_string()
        cmds = ['vlan %s' % vid, 'trunk group %s' % tg]
        func = function('add_trunk_group', vid, tg)
        self.eapi_positive_config_test(func, cmds)

    def test_remove_trunk_group(self):
        vid = random_vlan()
        tg = random_string()
        cmds = ['vlan %s' % vid, 'no trunk group %s' % tg]
        func = function('remove_trunk_group', vid, tg)
        self.eapi_positive_config_test(func, cmds)


if __name__ == '__main__':
    unittest.main()



import sys
import os
import unittest

sys.path.append(os.path.join(os.path.dirname(__file__), '../lib'))

from testlib import get_fixture, function
from testlib import EapiConfigUnitTest

import pyeapi.api.ospf


class TestApiOspf(EapiConfigUnitTest):

    def __init__(self, *args, **kwargs):
        super(TestApiOspf, self).__init__(*args, **kwargs)
        self.instance = pyeapi.api.ospf.instance(None)
        self.config = open(get_fixture('running_config.ospf')).read()

    def test_get_no_vrf(self):
        result = self.instance.get()
        keys = ['networks', 'ospf_process_id', 'vrf', 'redistributions',
                'router_id', 'shutdown']
        self.assertEqual(sorted(keys), sorted(result.keys()))
        self.assertEqual(result['vrf'], 'default')

    def test_get_with_vrf(self):
        result = self.instance.get(vrf='test')
        keys = ['networks', 'ospf_process_id', 'vrf', 'redistributions',
                'router_id', 'shutdown']
        self.assertEqual(sorted(keys), sorted(result.keys()))
        self.assertEqual(result['vrf'], 'test')

    def test_create(self):
        for ospf_id in ['65000', 65000]:
            func = function('create', ospf_id)
            cmds = 'router ospf {}'.format(ospf_id)
            self.eapi_positive_config_test(func, cmds)

    def test_create_with_vrf(self):
        for ospf_id in ['65000', 65000]:
            vrf_name = 'test'
            func = function('create', ospf_id, vrf_name)
            cmds = 'router ospf {} vrf {}'.format(ospf_id, vrf_name)
            self.eapi_positive_config_test(func, cmds)

    def test_create_invalid_id(self):
        for ospf_id in ['66000', 66000]:
            with self.assertRaises(ValueError):
                self.instance.create(ospf_id)

    def test_delete(self):
        func = function('delete')
        cmds = 'no router ospf 65000'
        self.eapi_positive_config_test(func, cmds)

    def test_add_network(self):
        func = function('add_network', '172.16.10.0', '24', '0')
        cmds = ['router ospf 65000', 'network 172.16.10.0/24 area 0']
        self.eapi_positive_config_test(func, cmds)

        func = function('add_network', '', '24', '0')
        self.eapi_exception_config_test(func, ValueError)

        func = function('add_network', '172.16.10.0', '', '0')
        self.eapi_exception_config_test(func, ValueError)

    def test_remove_network(self):
        func = function('remove_network', '172.16.10.0', '24', '0')
        cmds = ['router ospf 65000', 'no network 172.16.10.0/24 area 0']
        self.eapi_positive_config_test(func, cmds)

        func = function('remove_network', '', '24', '0')
        self.eapi_exception_config_test(func, ValueError)

        func = function('remove_network', '172.16.10.0', '', '0')
        self.eapi_exception_config_test(func, ValueError)

    def test_set_router_id(self):
        for state in ['config', 'negate', 'default']:
            rid = '1.1.1.1'
            if state == 'config':
                cmds = ['router ospf 65000', 'router-id 1.1.1.1']
                func = function('set_router_id', rid)
            elif state == 'negate':
                cmds = ['router ospf 65000', 'no router-id']
                func = function('set_router_id')
            elif state == 'default':
                cmds = ['router ospf 65000', 'default router-id']
                func = function('set_router_id', rid, True)
            self.eapi_positive_config_test(func, cmds)

        cmds = ['router ospf 65000', 'no router-id']
        func = function('set_router_id')
        self.eapi_positive_config_test(func, cmds)

    def test_set_shutdown(self):
        for state in ['config', 'negate', 'default']:
            if state == 'config':
                cmds = ['router ospf 65000', 'shutdown']
                func = function('set_shutdown')
            elif state == 'negate':
                cmds = ['router ospf 65000', 'no shutdown']
                func = function('set_no_shutdown')
            self.eapi_positive_config_test(func, cmds)

    def test_add_redistribution_no_route_map(self):
        for protocol in ['bgp', 'rip', 'static', 'connected', 'no-proto']:
            cmds = ['router ospf 65000', 'redistribute {}'.format(protocol)]
            func = function('add_redistribution', protocol)
            if protocol != 'no-proto':
                self.eapi_positive_config_test(func, cmds)
            else:
                self.eapi_exception_config_test(func, ValueError)

    def test_add_redistribution_with_route_map(self):
        for protocol in ['bgp', 'rip', 'static', 'connected']:
            cmds = ['router ospf 65000', 'redistribute {} route-map test'.format(protocol)]
            func = function('add_redistribution', protocol, 'test')
            if protocol != 'no-proto':
                self.eapi_positive_config_test(func, cmds)
            else:
                self.eapi_exception_config_test(func, ValueError)


    def test_delete_redistribution_no_route_map(self):
        for protocol in ['bgp', 'rip', 'static', 'connected', 'no-proto']:
            cmds = ['router ospf 65000', 'no redistribute {}'.format(protocol)]
            func = function('remove_redistribution', protocol)
            if protocol != 'no-proto':
                self.eapi_positive_config_test(func, cmds)
            else:
                self.eapi_exception_config_test(func, ValueError)


class TestApiNegOspf(EapiConfigUnitTest):

    def __init__(self, *args, **kwargs):
        super(TestApiNegOspf, self).__init__(*args, **kwargs)
        self.instance = pyeapi.api.ospf.instance(None)
        self.config = open(get_fixture('running_config.bgp')).read()

    def test_no_get(self):
        result = self.instance.get()
        self.assertEqual(None, result)

    def test_no_delete(self):
        result = self.instance.delete()
        self.assertTrue(result)


if __name__ == '__main__':
    unittest.main()



#
# Copyright (c) 2017, Arista Networks, Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
#   Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
#   Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
#
#   Neither the name of Arista Networks nor the names of its
#   contributors may be used to endorse or promote products derived from
#   this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL ARISTA NETWORKS
# BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
# BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
# WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
# OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
# IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
import sys
import os
import unittest

sys.path.append(os.path.join(os.path.dirname(__file__), '../lib'))

from testlib import get_fixture, function
from testlib import EapiConfigUnitTest

import pyeapi.api.vrfs


class TestApiVrfs(EapiConfigUnitTest):

    def __init__(self, *args, **kwargs):
        super(TestApiVrfs, self).__init__(*args, **kwargs)
        self.instance = pyeapi.api.vrfs.instance(None)
        self.config = open(get_fixture('running_config.vrf')).read()

    def test_get(self):
        result = self.instance.get('blah')
        vrf = dict(rd='10:10', vrf_name='blah', description='blah desc',
                   ipv4_routing=True, ipv6_routing=False)
        self.assertEqual(vrf, result)
        result2 = self.instance.get('test')
        vrf2 = dict(rd='200:500', vrf_name='test', description='!',
                    ipv4_routing=False, ipv6_routing=True)
        self.assertEqual(vrf2, result2)

    def test_get_not_configured(self):
        self.assertIsNone(self.instance.get('notthere'))

    def test_getall(self):
        result = self.instance.getall()
        self.assertIsInstance(result, dict)
        self.assertEqual(len(result), 3)

    def test_vrf_functions(self):
        for name in ['create', 'delete', 'default']:
            vrf_name = 'testvrf'
            if name == 'create':
                cmds = ['vrf definition %s' % vrf_name]
            elif name == 'delete':
                cmds = 'no vrf definition %s' % vrf_name
            elif name == 'default':
                cmds = 'default vrf definition %s' % vrf_name
            func = function(name, vrf_name)
            self.eapi_positive_config_test(func, cmds)

    def test_vrf_create_with_rd(self):
        vrf_name = 'testvrfrd'
        rd = '10:10'
        cmds = ['vrf definition %s' % vrf_name, 'rd %s' % rd]
        func = function('create', vrf_name, rd=rd)
        self.eapi_positive_config_test(func, cmds)

    def test_set_rd(self):
        vrf_name = 'testrdvrf'
        rd = '10:10'
        cmds = ['vrf definition %s' % vrf_name, 'rd %s' % rd]
        func = function('set_rd', vrf_name, rd)
        self.eapi_positive_config_test(func, cmds)

    def test_set_description(self):
        for state in ['config', 'negate', 'default']:
            vrf_name = 'testdescvrf'
            if state == 'config':
                description = 'testing'
                cmds = ['vrf definition %s' % vrf_name,
                        'description %s' % description]
                func = function('set_description', vrf_name, description)
                self.eapi_positive_config_test(func, cmds)
            elif state == 'negate':
                cmds = ['vrf definition %s' % vrf_name, 'no description']
                func = function('set_description', vrf_name, disable=True)
                self.eapi_positive_config_test(func, cmds)
            elif state == 'default':
                cmds = ['vrf definition %s' % vrf_name, 'default description']
                func = function('set_description', vrf_name, default=True)
                self.eapi_positive_config_test(func, cmds)

    def test_set_ipv4_routing(self):
        for state in ['config', 'negate', 'default']:
            vrf_name = 'testipv4vrf'
            if state == 'config':
                cmds = ['ip routing vrf %s' % vrf_name]
                func = function('set_ipv4_routing', vrf_name)
                self.eapi_positive_config_test(func, cmds)
            elif state == 'negate':
                cmds = ['no ip routing vrf %s' % vrf_name]
                func = function('set_ipv4_routing', vrf_name, disable=True)
                self.eapi_positive_config_test(func, cmds)
            elif state == 'default':
                cmds = ['default ip routing vrf %s' % vrf_name]
                func = function('set_ipv4_routing', vrf_name, default=True)
                self.eapi_positive_config_test(func, cmds)

    def test_set_ipv6_routing(self):
        for state in ['config', 'negate', 'default']:
            vrf_name = 'testipv6vrf'
            if state == 'config':
                cmds = ['ipv6 unicast-routing vrf %s' % vrf_name]
                func = function('set_ipv6_routing', vrf_name)
                self.eapi_positive_config_test(func, cmds)
            elif state == 'negate':
                cmds = ['no ipv6 unicast-routing vrf %s' % vrf_name]
                func = function('set_ipv6_routing', vrf_name, disable=True)
                self.eapi_positive_config_test(func, cmds)
            elif state == 'default':
                cmds = ['default ipv6 unicast-routing vrf %s' % vrf_name]
                func = function('set_ipv6_routing', vrf_name, default=True)
                self.eapi_positive_config_test(func, cmds)

    def test_set_interface(self):
        for state in ['config', 'negate', 'default']:
            vrf_name = 'testintvrf'
            interface = 'Ethernet1'
            if state == 'config':
                cmds = ['interface %s' % interface,
                        'vrf forwarding %s' % vrf_name]
                func = function('set_interface', vrf_name, interface)
                self.eapi_positive_config_test(func, cmds)
            elif state == 'negate':
                cmds = ['interface %s' % interface, 'no vrf forwarding']
                func = function('set_interface', vrf_name, interface,
                                disable=True)
                self.eapi_positive_config_test(func, cmds)
            elif state == 'default':
                cmds = ['interface %s' % interface, 'default vrf forwarding']
                func = function('set_interface', vrf_name, interface,
                                default=True)
                self.eapi_positive_config_test(func, cmds)


if __name__ == '__main__':
    unittest.main()



#
# Copyright (c) 2014, Arista Networks, Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
#   Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
#   Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
#
#   Neither the name of Arista Networks nor the names of its
#   contributors may be used to endorse or promote products derived from
#   this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL ARISTA NETWORKS
# BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
# BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
# WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
# OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
# IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
import os
import unittest

import sys
sys.path.append(os.path.join(os.path.dirname(__file__), '../lib'))

from systestlib import DutSystemTest, random_interface

class TestApiStpInterfaces(DutSystemTest):

    def test_get(self):
        for dut in self.duts:
            dut.config(['default interface Ethernet1'])
            keys = ['portfast', 'portfast_type', 'bpduguard']
            result = dut.api('stp').interfaces.get('Ethernet1')
            self.assertEqual(sorted(keys), sorted(result.keys()),
                             'dut=%s' % dut)

    def test_getall(self):
        for dut in self.duts:
            dut.config('default interface Et1-4')
            result = dut.api('stp').interfaces.getall()
            self.assertIsInstance(result, dict)

    def test_set_bpduguard_to_true(self):
        for dut in self.duts:
            intf = random_interface(dut)
            dut.config('default interface %s' % intf)
            resource = dut.api('stp').interfaces
            result = resource.set_bpduguard(intf, True)
            self.assertTrue(result, 'dut=%s' % dut)

    def test_set_bpdugard_to_false(self):
        for dut in self.duts:
            intf = random_interface(dut)
            dut.config(['default interface %s' % intf, 'interface %s' % intf,
                        'spanning-tree bpduguard enable'])
            resource = dut.api('stp').interfaces
            result = resource.set_bpduguard(intf, False)
            self.assertTrue(result, 'dut=%s' % dut)

    def test_set_bpdugard_to_default(self):
        for dut in self.duts:
            intf = random_interface(dut)
            dut.config(['default interface %s' % intf, 'interface %s' % intf,
                        'spanning-tree bpduguard enable'])
            resource = dut.api('stp').interfaces
            result = resource.set_bpduguard(intf, default=True)
            self.assertTrue(result, 'dut=%s' % dut)

    def test_set_bpdugard_to_no(self):
        for dut in self.duts:
            intf = random_interface(dut)
            dut.config(['default interface %s' % intf, 'interface %s' % intf,
                        'spanning-tree bpduguard enable'])
            resource = dut.api('stp').interfaces
            result = resource.set_bpduguard(intf, disable=True)
            self.assertTrue(result, 'dut=%s' % dut)

    def test_set_portfast_to_true(self):
        for dut in self.duts:
            intf = random_interface(dut)
            dut.config('default interface %s' % intf)
            resource = dut.api('stp').interfaces
            result = resource.set_portfast(intf, True)
            self.assertTrue(result, 'dut=%s' % dut)

    def test_set_portfast_to_false(self):
        for dut in self.duts:
            intf = random_interface(dut)
            dut.config(['default interface %s' % intf, 'interface %s' % intf,
                        'spanning-tree portfast'])
            resource = dut.api('stp').interfaces
            result = resource.set_portfast(intf, False)
            self.assertTrue(result, 'dut=%s' % dut)

    def test_set_portfast_to_default(self):
        for dut in self.duts:
            intf = random_interface(dut)
            dut.config(['default interface %s' % intf, 'interface %s' % intf,
                        'spanning-tree portfast'])
            resource = dut.api('stp').interfaces
            result = resource.set_portfast(intf, default=True)
            self.assertTrue(result, 'dut=%s' % dut)

    def test_set_portfast_to_no(self):
        for dut in self.duts:
            intf = random_interface(dut)
            dut.config(['default interface %s' % intf, 'interface %s' % intf,
                        'spanning-tree portfast'])
            resource = dut.api('stp').interfaces
            result = resource.set_portfast(intf, disable=True)
            self.assertTrue(result, 'dut=%s' % dut)

    def test_set_portfast_to_edge(self):
        for dut in self.duts:
            intf = random_interface(dut)
            dut.config('default interface %s' % intf)
            resource = dut.api('stp').interfaces
            result = resource.set_portfast_type(intf, 'edge')
            self.assertTrue(result, 'dut=%s' % dut)

    def test_set_portfast_to_network(self):
        for dut in self.duts:
            intf = random_interface(dut)
            dut.config(['default interface %s' % intf, 'interface %s' % intf,
                        'spanning-tree portfast'])
            resource = dut.api('stp').interfaces
            result = resource.set_portfast_type(intf, 'normal')
            self.assertTrue(result, 'dut=%s' % dut)


if __name__ == '__main__':
    unittest.main()



#
# Copyright (c) 2014, Arista Networks, Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
#   Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
#   Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
#
#   Neither the name of Arista Networks nor the names of its
#   contributors may be used to endorse or promote products derived from
#   this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL ARISTA NETWORKS
# BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
# BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
# WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
# OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
# IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
import os
import unittest
import time

import sys
sys.path.append(os.path.join(os.path.dirname(__file__), '../lib'))

from systestlib import DutSystemTest, random_interface


class TestResourceIpinterfaces(DutSystemTest):

    def test_get(self):
        for dut in self.duts:
            intf = random_interface(dut)
            dut.config(['default interface %s' % intf, 'interface %s' % intf,
                        'no switchport', 'ip address 99.98.99.99/24',
                        'mtu 1800'])
            result = dut.api('ipinterfaces').get(intf)
            values = dict(name=intf, address='99.98.99.99/24',
                          mtu=1800)
            self.assertEqual(values, result, 'dut=%s' % dut)

    def test_get_interface_wo_ip_adddress(self):
        for dut in self.duts:
            intf = random_interface(dut)
            dut.config(['default interface %s' % intf, 'interface %s' % intf,
                        'no switchport'])
            time.sleep(2)
            result = dut.api('ipinterfaces').get(intf)
            self.assertIsNone(result['address'])

    def test_getall(self):
        for dut in self.duts:
            result = dut.api('interfaces').getall()
            self.assertIsInstance(result, dict)
            for intf in ['Management1']:
                self.assertIn(intf, result)

    def test_create_and_return_true(self):
        for dut in self.duts:
            intf = random_interface(dut)
            dut.config('default interface %s' % intf)
            resource = dut.api('ipinterfaces')
            result = resource.create(intf)
            self.assertTrue(result, 'dut=%s' % dut)
            config = dut.run_commands('show running-config interfaces %s' %
                                      intf, 'text')
            self.assertIn('no switchport', config[0]['output'])
            dut.config('default interface %s' % intf)

    def test_delete_and_return_true(self):
        for dut in self.duts:
            intf = random_interface(dut)
            dut.config(['interface %s' % intf, 'ip address 199.1.1.1/24'])
            resource = dut.api('ipinterfaces')
            result = resource.delete(intf)
            self.assertTrue(result, 'dut=%s' % dut)
            config = dut.run_commands('show running-config interfaces %s' %
                                      intf, 'text')
            self.assertNotIn('ip address 199.1.1.1/24', config[0]['output'],
                             'dut=%s' % dut)
            dut.config('default interface %s' % intf)

    def test_set_address(self):
        for dut in self.duts:
            intf = random_interface(dut)
            dut.config(['default interface %s' % intf, 'interface %s' % intf,
                        'no switchport'])
            resource = dut.api('ipinterfaces')
            result = resource.set_address(intf, '111.111.111.111/24')
            self.assertTrue(result, 'dut=%s' % dut)
            config = dut.run_commands('show running-config interfaces %s' %
                                      intf, 'text')
            self.assertIn('ip address 111.111.111.111/24',
                          config[0]['output'], 'dut=%s' % dut)
            dut.config('default interface %s' % intf)

    def test_set_mtu(self):
        for dut in self.duts:
            intf = random_interface(dut)
            dut.config(['default interface %s' % intf, 'interface %s' % intf,
                        'ip address 111.111.111.111/24'])
            resource = dut.api('ipinterfaces')
            result = resource.set_mtu(intf, 2000)
            self.assertTrue(result, 'dut=%s' % dut)
            config = dut.run_commands('show running-config interfaces %s' %
                                      intf, 'text')
            self.assertIn('mtu 2000', config[0]['output'], 'dut=%s' % dut)
            dut.config('default interface %s' % intf)

    def test_set_mtu_value_as_string(self):
        for dut in self.duts:
            intf = random_interface(dut)
            dut.config(['default interface %s' % intf, 'interface %s' % intf,
                        'ip address 111.111.111.111/24'])
            resource = dut.api('ipinterfaces')
            result = resource.set_mtu(intf, '2000')
            self.assertTrue(result, 'dut=%s' % dut)
            config = dut.run_commands('show running-config interfaces %s' %
                                      intf, 'text')
            self.assertIn('mtu 2000', config[0]['output'], 'dut=%s' % dut)
            dut.config('default interface %s' % intf)


if __name__ == '__main__':
    unittest.main()
